@@ -3,6 +3,8 @@
 """Stream PCAP File Extractor
 ================================
 
+.. module:: pcapkit
+
 :mod:`pcapkit` is an independent open source library, using only
 `DictDumper`_ as its formatted output dumper.
 
@@ -61,7 +63,18 @@ different components.
   family extraction and representation.
 
 """
+###############################################################################
+# conda ``_extern`` module support
+
 import os
+import sys
+
+_extern = os.path.join(os.path.dirname(os.path.realpath(__file__)), '_extern')
+if os.path.exists(_extern):
+    sys.path.append(_extern)
+
+###############################################################################
+
 import warnings
 
 import tbtrim
@@ -74,7 +87,7 @@ from pcapkit.utilities.warnings import DevModeWarning
 if DEVMODE:
     warnings.showwarning('development mode enabled', DevModeWarning,
                          filename=__file__, lineno=0,
-                         line=f"PCAPKIT_DEVMODE={os.environ['PCAPKIT_DEVMODE']}")
+                         line=f"PCAPKIT_DEVMODE={os.environ.get('PCAPKIT_DEVMODE', '1')}")
 else:
     ROOT = os.path.dirname(os.path.realpath(__file__))
     tbtrim.set_trim_rule(lambda filename: ROOT in os.path.realpath(filename),
@@ -106,5 +119,9 @@ __all__ = [
 
     'TCP', 'UDP',                                           # Transport Layer
 
-    'FTP', 'HTTP',                                          # Application Layer
+    'FTP', 'FTP_DATA',                                      # Application Layer
+    'HTTP',
 ]
+
+#: version number
+__version__ = '1.0.2.post7'
