@@ -2,6 +2,8 @@
 """Command Line Tool
 =======================
 
+.. module:: pcapkit.__main__
+
 .. important::
 
    This module requires ``emoji`` package to be installed.
@@ -17,6 +19,9 @@ import argparse
 import sys
 from typing import TYPE_CHECKING
 
+from pcapkit import __version__
+from pcapkit.foundation.extraction import Extractor
+from pcapkit.interface import JSON, PLIST, TREE
 from pcapkit.utilities.compat import ModuleNotFoundError  # pylint: disable=redefined-builtin
 from pcapkit.utilities.exceptions import stacklevel
 from pcapkit.utilities.warnings import EmojiWarning, warn
@@ -27,15 +32,11 @@ except ModuleNotFoundError:
     warn("dependency package 'emoji' not found",
          EmojiWarning, stacklevel=stacklevel())
 
-from pcapkit.foundation.extraction import Extractor
-from pcapkit.interface import JSON, PLIST, TREE
+__all__ = ['main']
 
 if TYPE_CHECKING:
     from argparse import ArgumentParser
 
-#: version number
-__version__ = '0.16.3'
-
 
 def get_parser() -> 'ArgumentParser':
     """CLI argument parser."""
@@ -103,7 +104,7 @@ def main() -> 'int':
                           files=args.files, format=fmt,
                           layer=args.layer, protocol=args.protocol,
                           engine=args.engine, extension=args.auto_extension,
-                          verbose=args.verbose)
+                          verbose=args.verbose)  # type: ignore[var-annotated]
 
     if args.verbose:
         try:
