@@ -3,6 +3,8 @@
 """Library Index
 ===================
 
+.. module:: pcapkit.all
+
 :mod:`pcapkit` has defined various and numerous functions
 and classes, which have different features and purposes.
 To make a simple index for this library, :mod:`pcapkit.all`
@@ -52,9 +54,19 @@ __all__ = [
     'Info',                                                 # Info Class
     'ProtoChain',                                           # ProtoChain
     'VersionInfo',                                          # Version
+    'NumberField', 'Int32Field', 'UInt32Field',             # numeric protocol fields
+    'Int16Field', 'UInt16Field', 'Int64Field', 'UInt64Field',
+    'Int8Field', 'UInt8Field', 'EnumField',
+    'BytesField', 'StringField', 'BitField',                # text protocol fields
+    'PaddingField',
+    'ConditionalField', 'PayloadField', 'SchemaField',      # misc protocol fields
+    'ForwardMatchField', 'NoValueField',
+    'IPv4AddressField', 'IPv6AddressField',                 # IP address protocol fields
+    'IPv4InterfaceField', 'IPv6InterfaceField',
+    'ListField', 'OptionField',                             # container protocol fields
 
     # pcapkit.dumpkit
-    'PCAPIO',                                                 # PCAP Dumper
+    'PCAPIO',                                               # PCAP Dumper
     'NotImplementedIO',                                     # Simulated I/O
 
     # pcapkit.foundation
@@ -67,11 +79,16 @@ __all__ = [
 
     # pcapkit.foundation.registry
     'register_protocol',
-    'register_linktype', 'register_pcap',
+    'register_linktype', 'register_pcap', 'register_pcapng',
     'register_ethertype', 'register_transtype',
     'register_port', 'register_tcp_port', 'register_udp_port',
-    'register_output', 'register_extractor', 'register_traceflow',
+    'register_output',
+    'register_extractor_dumper', 'register_extractor_engine',
+    'register_traceflow',
+    'register_pcapng_block', 'register_pcapng_option', 'register_pcapng_secrets',
+    'register_pcapng_record',
     'register_hopopt', 'register_ipv6_opts', 'register_ipv6_route',
+    'register_ipv4', 'register_hip',
     'register_tcp', 'register_mptcp',
     'register_http',
 
@@ -92,11 +109,15 @@ __all__ = [
     'HIP', 'HOPOPT', 'IPv6_Frag', 'IPv6_Opts', 'IPv6_Route', 'MH',
                                                             # IPv6 Extension Header
     'TCP', 'UDP',                                           # Transport Layer
-    'FTP', 'HTTP',                                          # Application Layer
+    'FTP', 'FTP_DATA',                                      # Application Layer
+    'HTTP',
+    'Schema', 'schema',                                     # Protocol Schema
+    'Data', 'data',                                         # Protocol Data
 
     # pcapkit.toolkit
     'ipv4_reassembly', 'ipv6_reassembly', 'tcp_reassembly', 'tcp_traceflow',
-                                                            # default engine
+    'pcapng_ipv4_reassembly', 'pcapng_ipv6_reassembly', 'pcapng_tcp_reassembly', 'pcapng_tcp_traceflow',
+    'pcapng_block2frame',                                   # default engine
     'dpkt_ipv6_hdr_len', 'dpkt_packet2chain', 'dpkt_packet2dict',
     'dpkt_ipv4_reassembly', 'dpkt_ipv6_reassembly', 'dpkt_tcp_reassembly', 'dpkt_tcp_traceflow',
                                                             # DPKT engine
