@@ -3,6 +3,8 @@
 """:class:`~pcapkit.protocols.transport.tcp.TCP` Constant Enumerations
 =========================================================================
 
+.. module:: pcapkit.const.tcp
+
 This module contains all constant enumerations of
 :class:`~pcapkit.protocols.transport.tcp.TCP` implementations. Available
 enumerations include:
@@ -11,8 +13,10 @@ enumerations include:
 
    * - :class:`TCP_Checksum <pcapkit.const.tcp.checksum.Checksum>`
      - TCP Checksum [*]_
-   * - :class:`TCP_Option <pcapkit.vendor.tcp.option.Option>`
-     - TCP Option Kind Numbers [*]_
+   * - :class:`TCP_MPTCPOption <pcapkit.const.tcp.mp_tcp_option.MPTCPOption>`
+     - Multipath TCP options [*]_
+   * - :class:`TCP_Option <pcapkit.const.tcp.option.Option>`
+     - TCP Option Kind Numbers
 
 .. [*] https://www.iana.org/assignments/tcp-parameters/tcp-parameters.xhtml#tcp-parameters-2
 .. [*] https://www.iana.org/assignments/tcp-parameters/tcp-parameters.xhtml#tcp-parameters-1
