@@ -2,6 +2,8 @@
 """Info Class
 ================
 
+.. module:: pcapkit.corekit.infoclass
+
 :mod:`pcapkit.corekit.infoclass` contains :obj:`dict` like class
 :class:`~pcapkit.corekit.infoclass.Info` only, which is originally
 designed to work alike :func:`dataclasses.dataclass` as introduced
@@ -9,18 +11,133 @@ in :pep:`557`.
 
 """
 import collections.abc
+import enum
 import itertools
-from typing import TYPE_CHECKING, Generic, TypeVar
+from typing import TYPE_CHECKING, Generic, TypeVar, final
 
 from pcapkit.utilities.compat import Mapping
-from pcapkit.utilities.exceptions import UnsupportedCall
+from pcapkit.utilities.exceptions import UnsupportedCall, stacklevel
+from pcapkit.utilities.warnings import InfoWarning, warn
 
 if TYPE_CHECKING:
-    from typing import Any, Iterable, Iterator, NoReturn, Optional
+    from typing import Any, Iterable, Iterator, NoReturn, Optional, Type
 
-__all__ = ['Info']
+    from typing_extensions import Self
+
+__all__ = ['Info', 'info_final']
 
 VT = TypeVar('VT')
+ST = TypeVar('ST', bound='Type[Info]')
+
+
+class FinalisedState(enum.IntEnum):
+    """Finalised state."""
+
+    #: Not finalised.
+    NONE = enum.auto()
+    #: Base class.
+    BASE = enum.auto()
+    #: Finalised.
+    FINAL = enum.auto()
+
+
+def info_final(cls: 'ST', *, _finalised: 'bool' = True) -> 'ST':
+    """Finalise info class.
+
+    This decorator function is used to generate necessary
+    attributes and methods for the decorated :class:`Info`
+    class. It can be useful to reduce runtime generation
+    time as well as caching already generated attributes.
+
+    Args:
+        cls: Info class.
+        _finalised: Whether to make the info class as finalised.
+
+    Returns:
+        Finalised info class.
+
+    Notes:
+        The decorator should only be used on the *final*
+        class, otherwise, any subclasses derived from a
+        finalised info class will not be re-finalised.
+
+    :meta decorator:
+    """
+    if cls.__finalised__ == FinalisedState.FINAL:
+        warn(f'{cls.__name__}: info class has been finalised; now skipping',
+             InfoWarning, stacklevel=stacklevel())
+        return cls
+
+    temp = ['__map__', '__map_reverse__', '__builtin__', '__finalised__']
+    temp.extend(cls.__additional__)
+    for obj in cls.mro():
+        temp.extend(dir(obj))
+    cls.__builtin__ = set(temp)
+    cls.__excluded__.extend(cls.__builtin__)
+
+    # NOTE: We only generate ``__init__`` method for subclasses of the
+    # ``Info`` class, rather than itself, plus that such class does not
+    # override the ``__init__`` method of the meta class.
+    if '__init__' not in cls.__dict__ and cls is not Info:
+        args_ = []  # type: list[str]
+        dict_ = []  # type: list[str]
+
+        for cls_ in cls.mro():
+            # NOTE: We skip the ``Info`` class itself, to avoid superclass
+            # type annotations being considered.
+            if cls_ is Info:
+                break
+
+            # NOTE: We iterate in reversed order to ensure that the type
+            # annotations of the superclasses are considered first.
+            for key in reversed(cls_.__annotations__):
+                # NOTE: We skip duplicated annotations to avoid duplicate
+                # argument in function definition.
+                if key in args_:
+                    continue
+
+                args_.append(key)
+                dict_.append(f'{key}={key}')
+
+        # NOTE: We reverse the two lists such that the order of the
+        # arguments is the same as the order of the type annotations, i.e.,
+        # from the most base class to the most derived class.
+        args_.reverse()
+        dict_.reverse()
+
+        # NOTE: We only generate typed ``__init__`` method if only the class
+        # has type annotations from any of itself and its base classes.
+        if args_:
+            # NOTE: The following code is to make the ``__init__`` method work.
+            # It is inspired from the :func:`dataclasses._create_fn` function.
+            init_ = (
+                f'def __create_fn__():\n'
+                f'    def __init__(self, {", ".join(args_)}):\n'
+                f'        self.__update__({", ".join(dict_)})\n'
+                f'        self.__post_init__()\n'
+                f'    return __init__\n'
+            )
+        else:
+            init_ = (
+                'def __create_fn__():\n'
+                '    def __init__(self, dict_=None, **kwargs):\n'
+                '        self.__update__(dict_, **kwargs)\n'
+                '        self.__post_init__()\n'
+                '    return __init__\n'
+            )
+
+        ns = {}  # type: dict[str, Any]
+        exec(init_, None, ns)  # pylint: disable=exec-used # nosec
+
+        cls.__init__ = ns['__create_fn__']()  # type: ignore[misc]
+        cls.__init__.__qualname__ = f'{cls.__name__}.__init__'  # type: ignore[misc]
+
+    if not _finalised:
+        cls.__finalised__ = FinalisedState.BASE
+        return cls
+
+    cls.__finalised__ = FinalisedState.FINAL
+    return final(cls)
 
 
 class Info(Mapping[str, VT], Generic[VT]):
@@ -48,10 +165,18 @@ class Info(Mapping[str, VT], Generic[VT]):
         #: Mapping of name conflicts with builtin methods (transformed names to
         #: original names).
         __map_reverse__: 'dict[str, str]'
-        #： List of builtin methods.
+        #: List of builtin methods.
         __builtin__: 'set[str]'
 
-    def __new__(cls, *args: 'VT', **kwargs: 'VT') -> 'Info':  # pylint: disable=unused-argument
+    #: Flag for finalised class initialisation.
+    __finalised__: 'FinalisedState' = FinalisedState.NONE
+
+    #: List of additional built-in names.
+    __additional__: 'list[str]' = []
+    #: List of names to be excluded from :obj:`dict` conversion.
+    __excluded__: 'list[str]' = []
+
+    def __new__(cls, *args: 'VT', **kwargs: 'VT') -> 'Self':  # pylint: disable=unused-argument
         """Create a new instance.
 
         The class will try to automatically generate ``__init__`` method with
@@ -63,71 +188,21 @@ class Info(Mapping[str, VT], Generic[VT]):
             **kwargs: Arbitrary keyword arguments.
 
         """
-        cls.__map__ = {}
-        cls.__map_reverse__ = {}
-
-        temp = ['__map__', '__map_reverse__', '__builtin__']
-        for obj in cls.mro():
-            temp.extend(dir(obj))
-        cls.__builtin__ = set(temp)
-
-        # NOTE: We only generate ``__init__`` method for subclasses of the
-        # ``Info`` class, rather than itself, plus that such class does not
-        # override the ``__init__`` method of the meta class.
-        if '__init__' not in cls.__dict__ and cls is not Info:
-            args_ = []  # type: list[str]
-            dict_ = []  # type: list[str]
-
-            for cls_ in cls.mro():
-                # NOTE: We skip the ``Info`` class itself, to avoid superclass
-                # type annotations being considered.
-                if cls_ is Info:
-                    break
-
-                # NOTE: We iterate in reversed order to ensure that the type
-                # annotations of the superclasses are considered first.
-                for key in reversed(cls_.__annotations__):
-                    # NOTE: We skip duplicated annotations to avoid duplicate
-                    # argument in function definition.
-                    if key in args_:
-                        continue
-
-                    args_.append(key)
-                    dict_.append(f'{key}={key}')
-
-            # NOTE: We reverse the two lists such that the order of the
-            # arguments is the same as the order of the type annotations, i.e.,
-            # from the most base class to the most derived class.
-            args_.reverse()
-            dict_.reverse()
-
-            # NOTE: We only generate typed ``__init__`` method if only the class
-            # has type annotations from any of itself and its base classes.
-            if args_:
-                # NOTE: The following code is to make the ``__init__`` method work.
-                # It is inspired from the :func:`dataclasses._create_fn` function.
-                init_ = (
-                    f'def __create_fn__():\n'
-                    f'    def __init__(self, {", ".join(args_)}):\n'
-                    f'        self.__update__({", ".join(dict_)})\n'
-                    f'    return __init__\n'
-                )
-            else:
-                init_ = (
-                    'def __create_fn__():\n'
-                    '    def __init__(self, dict_=None, **kwargs):\n'
-                    '        self.__update__(dict_, **kwargs)\n'
-                    '    return __init__\n'
-                )
-
-            ns = {}  # type: dict[str, Any]
-            exec(init_, None, ns)  # pylint: disable=exec-used # nosec
-            cls.__init__ = ns['__create_fn__']()
-            cls.__init__.__qualname__ = f'{cls.__name__}.__init__'
-
+        if cls.__finalised__ == FinalisedState.NONE:
+            cls = info_final(cls, _finalised=False)
         self = super().__new__(cls)
+
+        # NOTE: We define the ``__map__`` and ``__map_reverse__`` attributes
+        # here under ``self`` to avoid them being considered as class variables
+        # and thus being shared by all instances.
+        super().__setattr__(self, '__map__', {})
+        super().__setattr__(self, '__map_reverse__', {})
+
         return self
 
+    def __post_init__(self) -> 'None':
+        """Customisation method to be called after initialisation."""
+
     def __update__(self, dict_: 'Optional[Mapping[str, VT] | Iterable[tuple[str, VT]]]' = None,
                    **kwargs: 'VT') -> 'None':
         # NOTE: Keys with the same names as the class's builtin methods will be
@@ -170,6 +245,9 @@ class Info(Mapping[str, VT], Generic[VT]):
     def __str__(self) -> 'str':
         temp = []  # type: list[str]
         for (key, value) in self.__dict__.items():
+            if key in self.__excluded__:
+                continue
+
             out_key = self.__map_reverse__.get(key, key)
             temp.append(f'{out_key}={value}')
         args = ', '.join(temp)
@@ -178,6 +256,9 @@ class Info(Mapping[str, VT], Generic[VT]):
     def __repr__(self) -> 'str':
         temp = []  # type: list[str]
         for (key, value) in self.__dict__.items():
+            if key in self.__excluded__:
+                continue
+
             out_key = self.__map_reverse__.get(key, key)
             if isinstance(value, Info):
                 temp.append(f'{out_key}={type(value).__name__}(...)')
@@ -193,8 +274,8 @@ class Info(Mapping[str, VT], Generic[VT]):
         for key in self.__dict__:
             yield self.__map_reverse__.get(key, key)
 
-    def __getitem__(self, key: 'str') -> 'VT':
-        key = self.__map__.get(key, key)
+    def __getitem__(self, name: 'str') -> 'VT':
+        key = self.__map__.get(name, name)
         return self.__dict__[key]
 
     def __setattr__(self, name: 'str', value: 'VT') -> 'NoReturn':
@@ -205,7 +286,7 @@ class Info(Mapping[str, VT], Generic[VT]):
 
     @classmethod
     def from_dict(cls, dict_: 'Optional[Mapping[str, VT] | Iterable[tuple[str, VT]]]' = None,
-                  **kwargs: 'VT') -> 'Info[VT]':
+                  **kwargs: 'VT') -> 'Self':
         r"""Create a new instance.
 
         * If ``dict_`` is present and has a ``.keys()`` method, then does:
@@ -237,6 +318,9 @@ class Info(Mapping[str, VT], Generic[VT]):
         """
         dict_ = {}  # type: dict[str, Any]
         for (key, value) in self.__dict__.items():
+            if key in self.__excluded__:
+                continue
+
             out_key = self.__map_reverse__.get(key, key)
             if isinstance(value, Info):
                 dict_[out_key] = value.to_dict()
