@@ -2,6 +2,8 @@
 """Multi-Mapping Dictionary
 ==============================
 
+.. module:: pcapkit.corekit.multidict
+
 :mod:`pcapkit.corekit.multidict` contains multi-mapping dictionary classes,
 which are used to store multiple mappings of the same key. The implementation
 is inspired and based on the `Werkzeug`_ project.
@@ -102,7 +104,7 @@ def iter_multi_items(mapping: 'Mapping[_KT, _VT | Iterable[_VT]] | Iterable[tupl
             else:
                 yield key, value
     else:
-        yield from mapping
+        yield from mapping  # type: ignore[misc]
 
 
 ###############################################################################
@@ -178,7 +180,7 @@ class MultiDict(dict, Generic[_KT, _VT]):
             lst = dict.__getitem__(self, key)
             if len(lst) > 0:
                 return lst[0]
-        raise MissingKeyError(key)
+        raise MissingKeyError(key, quiet=True)
 
     def __setitem__(self, key: '_KT', value: '_VT') -> 'None':
         """Like :meth:`add` but removes an existing key first.
@@ -509,7 +511,7 @@ class OrderedMultiDict(MultiDict[_KT, _VT]):
     def __getitem__(self, key: '_KT') -> '_VT':
         if key in self:
             return dict.__getitem__(self, key)[0].value
-        raise MissingKeyError(key)
+        raise MissingKeyError(key, quiet=True)
 
     def __setitem__(self, key: '_KT', value: '_VT') -> 'None':
         self.poplist(key)
