@@ -2,6 +2,8 @@
 """Protocol Chain
 ====================
 
+.. module:: pcapkit.corekit.protochain
+
 :mod:`pcapkit.corekit.protochain` contains special protocol
 collection class :class:`~pcapkit.corekit.protochain.ProtoChain`.
 
@@ -16,6 +18,8 @@ from pcapkit.utilities.exceptions import IndexNotFound
 if TYPE_CHECKING:
     from typing import Iterator, Optional, Type
 
+    from typing_extensions import Self
+
     from pcapkit.protocols.protocol import Protocol
 
 __all__ = ['ProtoChain']
@@ -51,7 +55,7 @@ class ProtoChain(collections.abc.Sequence):
     ##########################################################################
 
     @classmethod
-    def from_list(cls, data: 'list[Protocol | Type[Protocol]]') -> 'ProtoChain':
+    def from_list(cls, data: 'list[Protocol | Type[Protocol]]') -> 'Self':
         """Create a protocol chain from a list.
 
         Args:
@@ -176,7 +180,15 @@ class ProtoChain(collections.abc.Sequence):
         return ':'.join(map(lambda p: p[0], self.__data__))
 
     def __contains__(self, name: 'str | Protocol | Type[Protocol]') -> 'bool':  # type: ignore[override]
-        """Returns if ``name`` is in the chain."""
+        """Returns if ``name`` is in the chain.
+
+        Args:
+            name: Name to search.
+
+        Returns:
+            Whether ``name`` is in the chain.
+
+        """
         from pcapkit.protocols.protocol import Protocol  # pylint: disable=import-outside-toplevel
         comp = Protocol.expand_comp(name)
 
