@@ -2,6 +2,8 @@
 """Base Class
 ================
 
+.. module:: pcapkit.foundation.reassembly.reassembly
+
 :mod:`pcapkit.foundation.reassembly.reassembly` contains
 :class:`~pcapkit.foundation.reassembly.reassembly.Reassembly` only,
 which is an abstract base class for all reassembly classes,
@@ -15,6 +17,8 @@ from typing import TYPE_CHECKING, Generic, TypeVar
 if TYPE_CHECKING:
     from typing import Any, Optional, Type
 
+    from typing_extensions import Self
+
     from pcapkit.corekit.infoclass import Info
     from pcapkit.protocols.protocol import Protocol
 
@@ -38,6 +42,7 @@ class Reassembly(Generic[PT, DT, IT, BT], metaclass=abc.ABCMeta):
             implemented) when submit
 
     """
+
     _strflg: 'bool'
     _newflg: 'bool'
 
@@ -62,15 +67,7 @@ class Reassembly(Generic[PT, DT, IT, BT], metaclass=abc.ABCMeta):
             self.__cached__.clear()
             self._newflg = False
 
-        if False:
-            cached = NotImplemented
-
-        def _walrus_wrapper_cached_f6cceac7477e487cb1c2b89475ada686(expr):
-            """Wrapper function for assignment expression."""
-            nonlocal cached
-            cached = expr
-            return cached
-        if (_walrus_wrapper_cached_f6cceac7477e487cb1c2b89475ada686(self.__cached__.get('count'))) is not None:
+        if (cached := self.__cached__.get('count')) is not None:
             return cached
 
         ret = len(self.datagram)
@@ -141,15 +138,7 @@ class Reassembly(Generic[PT, DT, IT, BT], metaclass=abc.ABCMeta):
             self.__cached__.clear()
             self._newflg = False
 
-        if False:
-            cached = NotImplemented
-
-        def _walrus_wrapper_cached_0e28120f85a7482f972285b39b09cc9a(expr):
-            """Wrapper function for assignment expression."""
-            nonlocal cached
-            cached = expr
-            return cached
-        if (_walrus_wrapper_cached_0e28120f85a7482f972285b39b09cc9a(self.__cached__.get('fetch'))) is not None:
+        if (cached := self.__cached__.get('fetch')) is not None:
             return cached
 
         temp_dtgram = []  # type: list[DT]
@@ -195,7 +184,7 @@ class Reassembly(Generic[PT, DT, IT, BT], metaclass=abc.ABCMeta):
     # Data models.
     ##########################################################################
 
-    def __new__(cls, *args: 'Any', **kwargs: 'Any') -> 'Reassembly[PT, DT, IT, BT]':  # pylint: disable=unused-argument
+    def __new__(cls, *args: 'Any', **kwargs: 'Any') -> 'Self':  # pylint: disable=unused-argument
         self = super().__new__(cls)
 
         # NOTE: Assign this attribute after ``__new__`` to avoid shared memory
