@@ -2,6 +2,8 @@
 """Auxiliary Interface
 =========================
 
+.. module:: pcapkit.interface.misc
+
 :mod:`pcapkit.interface.misc` contains miscellaneous
 user interface functions, classes, etc., which are
 generally provided per user's requests.
@@ -10,9 +12,9 @@ generally provided per user's requests.
 import sys
 from typing import TYPE_CHECKING
 
-from pcapkit.corekit.infoclass import Info
+from pcapkit.corekit.infoclass import Info, info_final
 from pcapkit.foundation.extraction import Extractor
-from pcapkit.foundation.reassembly.tcp import TCP_Reassembly
+from pcapkit.foundation.reassembly.tcp import TCP as TCP_Reassembly
 from pcapkit.utilities.exceptions import stacklevel
 from pcapkit.utilities.warnings import EngineWarning, warn
 
@@ -21,7 +23,7 @@ if TYPE_CHECKING:
 
     from typing_extensions import Literal
 
-    from pcapkit.protocols.misc.pcap.frame import Frame
+    from pcapkit.foundation.extraction import Packet
 
     ByteOrder = Literal['little', 'big']
     Formats = Literal['pcap', 'json', 'tree', 'plist']
@@ -34,18 +36,19 @@ __all__ = ['follow_tcp_stream']
 ###############################################################################
 
 
+@info_final
 class Stream(Info):
     """Data model for TCP streams."""
 
     #: Output filename.
     filename: 'Optional[str]'
     #: Packet list.
-    packets: 'tuple[Frame, ...]'
+    packets: 'tuple[Packet, ...]'
     #: TCP conversation.
     conversations: 'tuple[bytes | tuple[bytes, ...], ...]'
 
     if TYPE_CHECKING:
-        def __init__(self, filename: 'Optional[str]', packets: 'tuple[Frame, ...]', conversations: 'tuple[bytes | tuple[bytes, ...], ...]') -> 'None': ...  # pylint: disable=unused-argument,super-init-not-called,multiple-statements,line-too-long
+        def __init__(self, filename: 'Optional[str]', packets: 'tuple[Packet, ...]', conversations: 'tuple[bytes | tuple[bytes, ...], ...]') -> 'None': ...  # pylint: disable=unused-argument,super-init-not-called,multiple-statements,line-too-long
 
 
 def follow_tcp_stream(fin: 'Optional[str]' = None, verbose: 'bool' = False,              # Extrator options
@@ -78,23 +81,23 @@ def follow_tcp_stream(fin: 'Optional[str]' = None, verbose: 'bool' = False,
                            store=True, files=False, nofile=True, verbose=verbose, engine=engine,
                            layer=None, protocol=None, ip=False, ipv4=False, ipv6=False, tcp=False,
                            strict=False, trace=True, trace_fout=fout, trace_format=format,
-                           trace_byteorder=byteorder, trace_nanosecond=nanosecond)
+                           trace_byteorder=byteorder, trace_nanosecond=nanosecond)  # type: ignore[var-annotated]
 
     fallback = False
-    if extraction.engine == 'dpkt':
+    if extraction.engine == 'dpkt':  # type: ignore[comparison-overlap]
         from pcapkit.toolkit.dpkt import tcp_reassembly  # pylint: disable=import-outside-toplevel
-    elif extraction.engine == 'scapy':
-        from pcapkit.toolkit.scapy import tcp_reassembly  # type: ignore[no-redef] # isort: skip # pylint: disable=import-outside-toplevel
+    elif extraction.engine == 'scapy':  # type: ignore[comparison-overlap]
+        from pcapkit.toolkit.scapy import tcp_reassembly  # isort: skip # pylint: disable=import-outside-toplevel
     else:
-        from pcapkit.toolkit.default import tcp_reassembly  # type: ignore[no-redef] # isort: skip # pylint: disable=import-outside-toplevel
+        from pcapkit.toolkit.pcap import tcp_reassembly  # type: ignore[assignment] # isort: skip # pylint: disable=import-outside-toplevel
         fallback = True
 
     streams = []  # type: list[Stream]
     frames = extraction.frame
-    for stream in extraction.trace:
+    for stream in extraction.trace.tcp:
         reassembly = TCP_Reassembly(strict=False)
 
-        packets = []  # type: list[Frame]
+        packets = []  # type: list[Packet]
         for index in stream.index:
             frame = frames[index-1]
             packets.append(frame)
