@@ -5,10 +5,10 @@ from typing import TYPE_CHECKING
 
 from pcapkit.const.ipv4.option_class import OptionClass
 from pcapkit.const.ipv4.tos_del import ToSDelay
-from pcapkit.corekit.infoclass import Info
+from pcapkit.corekit.infoclass import info_final
+from pcapkit.protocols.data.data import Data
 
 if TYPE_CHECKING:
-    from datetime import datetime as dt_type
     from datetime import timedelta
     from ipaddress import IPv4Address
     from typing import Any, Optional
@@ -39,10 +39,12 @@ __all__ = [
     'ESECOption', 'RROption', 'SIDOption',
     'SSROption', 'MTUPOption', 'MTUROption',
     'TROption', 'RTRALTOption', 'QSOption',
+    'QuickStartRequestOption', 'QuickStartReportOption',
 ]
 
 
-class ToSField(Info):
+@info_final
+class ToSField(Data):
     """Data model for IPv4 ToS fields.
 
     Important:
@@ -69,10 +71,11 @@ class ToSField(Info):
         # Thus, we directly inject the information into the annotations.
         self.__annotations__['del'] = ToSDelay  # pylint: disable=no-member
 
-        return self  # type: ignore[return-value]
+        return self
 
 
-class Flags(Info):
+@info_final
+class Flags(Data):
     """Data model for IPv4 Flags."""
 
     #: Don't fragment.
@@ -84,7 +87,8 @@ class Flags(Info):
         def __init__(self, df: 'bool', mf: 'bool') -> 'None': ...  # pylint: disable=unused-argument,super-init-not-called,multiple-statements,line-too-long,redefined-builtin
 
 
-class IPv4(Info):
+@info_final
+class IPv4(Data):
     """Data model for IPv4 packet."""
 
     #: Version.
@@ -118,7 +122,8 @@ class IPv4(Info):
         def __init__(self, version: 'Literal[4]', hdr_len: 'int', tos: 'ToSField', len: 'int', id: 'int', flags: 'Flags', offset: 'int', ttl: 'timedelta', protocol: 'TransType', checksum: 'bytes', src: 'IPv4Address', dst: 'IPv4Address') -> 'None': ...  # pylint: disable=unused-argument,super-init-not-called,multiple-statements,line-too-long,redefined-builtin
 
 
-class OptionType(Info):
+@info_final
+class OptionType(Data):
     """Data model for IPv4 option type data."""
 
     #: Change flag.
@@ -135,10 +140,10 @@ class OptionType(Info):
         # Thus, we directly inject the information into the annotations.
         self.__annotations__['class'] = OptionClass  # pylint: disable=no-member
 
-        return self  # type: ignore[return-value]
+        return self
 
 
-class Option(Info):
+class Option(Data):
     """Data model for IPv4 options."""
 
     #: Option code.
@@ -149,6 +154,7 @@ class Option(Info):
     type: 'OptionType'
 
 
+@info_final
 class UnassignedOption(Option):
     """Data model for IPv4 unassigned option."""
 
@@ -159,6 +165,7 @@ class UnassignedOption(Option):
         def __init__(self, code: 'OptionNumber', length: 'int', type: 'OptionType', data: 'bytes') -> 'None': ...  # pylint: disable=unused-argument,super-init-not-called,multiple-statements,line-too-long,redefined-builtin
 
 
+@info_final
 class EOOLOption(Option):
     """Data model for IPv4 End of Option List (``EOOL``) option."""
 
@@ -166,6 +173,7 @@ class EOOLOption(Option):
         def __init__(self, code: 'OptionNumber', length: 'int', type: 'OptionType') -> 'None': ...  # pylint: disable=unused-argument,super-init-not-called,multiple-statements,line-too-long,redefined-builtin
 
 
+@info_final
 class NOPOption(Option):
     """Data model for IPv4 No Operation (``NOP``) option."""
 
@@ -173,30 +181,33 @@ class NOPOption(Option):
         def __init__(self, code: 'OptionNumber', length: 'int', type: 'OptionType') -> 'None': ...  # pylint: disable=unused-argument,super-init-not-called,multiple-statements,line-too-long,redefined-builtin
 
 
+@info_final
 class SECOption(Option):
     """Data model for IPv4 Security (``SEC``) option."""
 
     #: Classification level.
     level: 'ClassificationLevel'
     #: Protection authority flags.
-    flags: 'Optional[OrderedMultiDict[ProtectionAuthority, bool]]'
+    flags: 'tuple[ProtectionAuthority, ...]'
 
     if TYPE_CHECKING:
-        def __init__(self, code: 'OptionNumber', length: 'int', type: 'OptionType', level: 'ClassificationLevel', flags: 'Optional[OrderedMultiDict[ProtectionAuthority, bool]]') -> 'None': ...  # pylint: disable=unused-argument,super-init-not-called,multiple-statements,line-too-long,redefined-builtin
+        def __init__(self, code: 'OptionNumber', length: 'int', type: 'OptionType', level: 'ClassificationLevel', flags: 'tuple[ProtectionAuthority, ...]') -> 'None': ...  # pylint: disable=unused-argument,super-init-not-called,multiple-statements,line-too-long,redefined-builtin
 
 
+@info_final
 class LSROption(Option):
     """Data model for IPv4 Loose Source Route (``LSR``) option."""
 
     #: Pointer.
     pointer: 'int'
     #: Route.
-    route: 'Optional[tuple[IPv4Address, ...]]'
+    route: 'tuple[IPv4Address, ...]'
 
     if TYPE_CHECKING:
-        def __init__(self, code: 'OptionNumber', length: 'int', type: 'OptionType', pointer: 'int', route: 'Optional[tuple[IPv4Address, ...]]') -> 'None': ...  # pylint: disable=unused-argument,super-init-not-called,multiple-statements,line-too-long,redefined-builtin
+        def __init__(self, code: 'OptionNumber', length: 'int', type: 'OptionType', pointer: 'int', route: 'tuple[IPv4Address, ...]') -> 'None': ...  # pylint: disable=unused-argument,super-init-not-called,multiple-statements,line-too-long,redefined-builtin
 
 
+@info_final
 class TSOption(Option):
     """Data model for IPv4 Time Stamp (``TS``) option."""
 
@@ -207,24 +218,26 @@ class TSOption(Option):
     #: Flag.
     flag: 'TSFlag'
     #: Timestamp data.
-    timestamp: 'Optional[bytes | tuple[dt_type, ...] | OrderedMultiDict[IPv4Address, dt_type]]'
+    timestamp: 'tuple[timedelta | int, ...] | OrderedMultiDict[IPv4Address, timedelta | int]'
 
     if TYPE_CHECKING:
-        def __init__(self, code: 'OptionNumber', length: 'int', type: 'OptionType', pointer: 'int', overflow: 'int', flag: 'TSFlag', timestamp: 'Optional[bytes | tuple[dt_type, ...] | OrderedMultiDict[IPv4Address, dt_type]]') -> 'None': ...  # pylint: disable=unused-argument,super-init-not-called,multiple-statements,line-too-long,redefined-builtin
+        def __init__(self, code: 'OptionNumber', length: 'int', type: 'OptionType', pointer: 'int', overflow: 'int', flag: 'TSFlag', timestamp: 'tuple[timedelta | int, ...] | OrderedMultiDict[IPv4Address, timedelta | int]') -> 'None': ...  # pylint: disable=unused-argument,super-init-not-called,multiple-statements,line-too-long,redefined-builtin
 
 
+@info_final
 class ESECOption(Option):
     """Data model for IPv4 Extended Security (``ESEC``) option."""
 
-    #: Classification level.
-    level: 'ClassificationLevel'
-    #: Protection authority flags.
-    flags: 'Optional[OrderedMultiDict[ProtectionAuthority, bool]]'
+    #: Additional security information format code.
+    format: 'int'
+    #: Additional security information.
+    info: 'bytes'
 
     if TYPE_CHECKING:
-        def __init__(self, code: 'OptionNumber', length: 'int', type: 'OptionType', level: 'ClassificationLevel', flags: 'Optional[OrderedMultiDict[ProtectionAuthority, bool]]') -> 'None': ...  # pylint: disable=unused-argument,super-init-not-called,multiple-statements,line-too-long,redefined-builtin
+        def __init__(self, code: 'OptionNumber', length: 'int', type: 'OptionType', format: 'int', info: 'bytes') -> 'None': ...  # pylint: disable=unused-argument,super-init-not-called,multiple-statements,line-too-long,redefined-builtin
 
 
+@info_final
 class RROption(Option):
     """Data model for IPv4 Record Route (``RR``) option."""
 
@@ -237,6 +250,7 @@ class RROption(Option):
         def __init__(self, code: 'OptionNumber', length: 'int', type: 'OptionType', pointer: 'int', route: 'Optional[tuple[IPv4Address, ...]]') -> 'None': ...  # pylint: disable=unused-argument,super-init-not-called,multiple-statements,line-too-long,redefined-builtin
 
 
+@info_final
 class SIDOption(Option):
     """Data model for IPv4 Stream ID (``SID``) option."""
 
@@ -247,6 +261,7 @@ class SIDOption(Option):
         def __init__(self, code: 'OptionNumber', length: 'int', type: 'OptionType', sid: 'int') -> 'None': ...  # pylint: disable=unused-argument,super-init-not-called,multiple-statements,line-too-long,redefined-builtin
 
 
+@info_final
 class SSROption(Option):
     """Data model for IPv4 Strict Source Route (``SSR``) option."""
 
@@ -259,6 +274,7 @@ class SSROption(Option):
         def __init__(self, code: 'OptionNumber', length: 'int', type: 'OptionType', pointer: 'int', route: 'Optional[tuple[IPv4Address, ...]]') -> 'None': ...  # pylint: disable=unused-argument,super-init-not-called,multiple-statements,line-too-long,redefined-builtin
 
 
+@info_final
 class MTUPOption(Option):
     """Data model for IPv4 MTU Probe (``MTUP``) option."""
 
@@ -269,6 +285,7 @@ class MTUPOption(Option):
         def __init__(self, code: 'OptionNumber', length: 'int', type: 'OptionType', mtu: 'int') -> 'None': ...  # pylint: disable=unused-argument,super-init-not-called,multiple-statements,line-too-long,redefined-builtin
 
 
+@info_final
 class MTUROption(Option):
     """Data model for IPv4 MTU Reply (``MTUR``) option."""
 
@@ -279,6 +296,7 @@ class MTUROption(Option):
         def __init__(self, code: 'OptionNumber', length: 'int', type: 'OptionType', mtu: 'int') -> 'None': ...  # pylint: disable=unused-argument,super-init-not-called,multiple-statements,line-too-long,redefined-builtin
 
 
+@info_final
 class TROption(Option):
     """Data model for IPv4 Traceroute (``TR``) option.
 
@@ -303,9 +321,10 @@ class TROption(Option):
         # Thus, we directly inject the information into the annotations.
         self.__annotations__['return'] = int  # pylint: disable=no-member
 
-        return self  # type: ignore[return-value]
+        return self
 
 
+@info_final
 class RTRALTOption(Option):
     """Data model for IPv4 Router Alert (``RTRALT``) option."""
 
@@ -323,10 +342,27 @@ class QSOption(Option):
     func: 'QSFunction'
     #: Rate request/report.
     rate: 'int'
+
+
+@info_final
+class QuickStartRequestOption(QSOption):
+    """Data model for IPv4 Quick Start request option."""
+
     #: TTL.
-    ttl: 'Optional[timedelta]'
-    #: Nounce.
-    nounce: 'int'
+    ttl: 'timedelta'
+    #: Nonce.
+    nonce: 'int'
 
     if TYPE_CHECKING:
-        def __init__(self, code: 'OptionNumber', length: 'int', type: 'OptionType', func: 'QSFunction', rate: 'int', ttl: 'Optional[timedelta]', nounce: 'int') -> 'None': ...  # pylint: disable=super-init-not-called,unused-argument,redefined-builtin,multiple-statements,line-too-long
+         def __init__(self, code: 'OptionNumber', length: 'int', type: 'OptionType', func: 'QSFunction', rate: 'int', ttl: 'timedelta', nonce: 'int') -> 'None': ...  # pylint: disable=unused-argument,super-init-not-called,multiple-statements,line-too-long,redefined-builtin
+
+
+@info_final
+class QuickStartReportOption(QSOption):
+    """Data model for IPv4 Quick Start report of approved rate option."""
+
+    #: Nonce.
+    nonce: 'int'
+
+    if TYPE_CHECKING:
+        def __init__(self, code: 'OptionNumber', length: 'int', type: 'OptionType', func: 'QSFunction', rate: 'int', nonce: 'int') -> 'None': ...  # pylint: disable=unused-argument,super-init-not-called,multiple-statements,line-too-long,redefined-builtin
