@@ -3,7 +3,8 @@
 
 from typing import TYPE_CHECKING
 
-from pcapkit.corekit.infoclass import Info
+from pcapkit.corekit.infoclass import info_final
+from pcapkit.protocols.data.data import Data
 
 if TYPE_CHECKING:
     from ipaddress import IPv4Address, IPv6Address
@@ -15,7 +16,8 @@ if TYPE_CHECKING:
 __all__ = ['Address', 'Type', 'ARP']
 
 
-class Address(Info):
+@info_final
+class Address(Data):
     """Data model for ARP addresses."""
 
     #: Hardware address.
@@ -27,7 +29,8 @@ class Address(Info):
         def __init__(self, hardware: 'str', protocol: 'str | IPv4Address | IPv6Address') -> 'None': ...  # pylint: disable=line-too-long,super-init-not-called,unused-argument,multiple-statements
 
 
-class Type(Info):
+@info_final
+class Type(Data):
     """Data model for ARP type."""
 
     #: Hardware type.
@@ -39,13 +42,14 @@ class Type(Info):
         def __init__(self, hardware: 'Hardware', protocol: 'EtherType | str') -> 'None': ...  # pylint: disable=line-too-long,super-init-not-called,unused-argument,multiple-statements
 
 
-class ARP(Info):
+@info_final
+class ARP(Data):
     """Data model for ARP packet."""
 
     #: Hardware type.
     htype: 'Hardware'
     #: Protocol type.
-    ptype: 'EtherType | str'
+    ptype: 'EtherType'
     #: Hardware address length.
     hlen: 'int'
     #: Protocol address length.
@@ -64,6 +68,6 @@ class ARP(Info):
     len: 'int'
 
     if TYPE_CHECKING:
-        def __init__(self, htype: 'Hardware', ptype: 'EtherType | str', hlen: 'int', plen: 'int',  # pylint: disable=line-too-long,super-init-not-called,unused-argument,multiple-statements
+        def __init__(self, htype: 'Hardware', ptype: 'EtherType', hlen: 'int', plen: 'int',  # pylint: disable=line-too-long,super-init-not-called,unused-argument,multiple-statements
                      oper: 'Operation', sha: 'str', spa: 'str | IPv4Address | IPv6Address',  # pylint: disable=unused-argument
                      tha: 'str', tpa: 'str | IPv4Address | IPv6Address', len: 'int') -> 'None': ...  # pylint: disable=unused-argument,redefined-builtin
