@@ -1,9 +1,10 @@
 # -*- coding: utf-8 -*-
-"""data modules for frame header of PCAP file"""
+"""data models for frame header of PCAP file"""
 
 from typing import TYPE_CHECKING
 
-from pcapkit.corekit.infoclass import Info
+from pcapkit.corekit.infoclass import info_final
+from pcapkit.protocols.data.data import Data
 
 if TYPE_CHECKING:
     from datetime import datetime
@@ -12,7 +13,8 @@ if TYPE_CHECKING:
 __all__ = ['Frame', 'FrameInfo']
 
 
-class FrameInfo(Info):
+@info_final
+class FrameInfo(Data):
     """Frame metadata information."""
 
     #: Timestamp seconds.
@@ -28,7 +30,8 @@ class FrameInfo(Info):
         def __init__(self, ts_sec: 'int', ts_usec: 'int', incl_len: 'int', orig_len: 'int') -> 'None': ...  # pylint: disable=unused-argument,multiple-statements,super-init-not-called
 
 
-class Frame(Info):
+@info_final
+class Frame(Data):
     """Frame header of PCAP file."""
 
     #: Metadata information.
