@@ -1,16 +1,18 @@
 # -*- coding: utf-8 -*-
-"""data modules for root protocol"""
+"""data models for root protocol"""
 
 from typing import TYPE_CHECKING
 
-from pcapkit.corekit.infoclass import Info
+from pcapkit.corekit.infoclass import info_final
+from pcapkit.protocols.data.data import Data
 
 __all__ = [
     'Packet',
 ]
 
 
-class Packet(Info):
+@info_final
+class Packet(Data):
     """Header and payload data."""
 
     #: packet header
