@@ -3,26 +3,26 @@
 
 from typing import TYPE_CHECKING
 
-from pcapkit.corekit.infoclass import Info
+from pcapkit.corekit.infoclass import info_final
+from pcapkit.protocols.data.data import Data
 
 if TYPE_CHECKING:
     from datetime import timedelta
     from ipaddress import IPv4Address, IPv6Address
     from typing import Optional, Union
 
-    from typing_extensions import Literal
-
     from pcapkit.const.tcp.checksum import Checksum
     from pcapkit.const.tcp.mp_tcp_option import MPTCPOption
     from pcapkit.const.tcp.option import Option as OptionNumber
     from pcapkit.corekit.multidict import OrderedMultiDict
+    from pcapkit.protocols.transport.tcp import Flags as TCP_Flags
 
     IPAddress = Union[IPv4Address, IPv6Address]
 
 __all__ = [
     'TCP',
 
-    'Flags',
+    'Flags', 'SACKBlock',
 
     'Option',
     'UnassignedOption', 'EndOfOptionList', 'NoOperation', 'MaximumSegmentSize', 'WindowScale',
@@ -31,7 +31,7 @@ __all__ = [
     'AlternateChecksumData', 'MD5Signature', 'QuickStartResponse', 'UserTimeout',
     'Authentication', 'FastOpenCookie',
 
-    'MPTCPCapableFlag', 'MPTCPDSSFlag',
+    'MPTCPCapableFlag',
 
     'MPTCP',
     'MPTCPUnknown', 'MPTCPCapable', 'MPTCPDSS', 'MPTCPAddAddress', 'MPTCPRemoveAddress',
@@ -42,7 +42,8 @@ __all__ = [
 ]
 
 
-class Flags(Info):
+@info_final
+class Flags(Data):
     """Data model for TCP flags."""
 
     #: ECN-nonce concealment protection.
@@ -68,7 +69,8 @@ class Flags(Info):
         def __init__(self, ns: 'bool', cwr: 'bool', ece: 'bool', urg: 'bool', ack: 'bool', psh: 'bool', rst: 'bool', syn: 'bool', fin: 'bool') -> 'None': ...  # pylint: disable=unused-argument,super-init-not-called,multiple-statements,line-too-long,redefined-builtin
 
 
-class TCP(Info):
+@info_final
+class TCP(Data):
     """Data model for TCP packet."""
 
     #: Source port.
@@ -91,12 +93,15 @@ class TCP(Info):
     urgent_pointer: 'int'
 
     if TYPE_CHECKING:
+        #: TCP options.
         options: 'OrderedMultiDict[OptionNumber, Option]'
+        #: Connection control flags.
+        connection: 'TCP_Flags'
 
         def __init__(self, srcport: 'int', dstport: 'int', seq: 'int', ack: 'int', hdr_len: 'int', flags: 'Flags', window_size: 'int', checksum: 'bytes', urgent_pointer: 'int') -> 'None': ...  # pylint: disable=unused-argument,super-init-not-called,multiple-statements,line-too-long,redefined-builtin
 
 
-class Option(Info):
+class Option(Data):
     """Data model for TCP options."""
 
     #: Option kind.
@@ -105,6 +110,7 @@ class Option(Info):
     length: 'int'
 
 
+@info_final
 class UnassignedOption(Option):
     """Data model for unassigned TCP option."""
 
@@ -115,6 +121,7 @@ class UnassignedOption(Option):
         def __init__(self, kind: 'OptionNumber', length: 'int', data: 'bytes') -> 'None': ...  # pylint: disable=unused-argument,super-init-not-called,multiple-statements,line-too-long,redefined-builtin
 
 
+@info_final
 class EndOfOptionList(Option):
     """Data model for TCP end of option list option."""
 
@@ -122,6 +129,7 @@ class EndOfOptionList(Option):
         def __init__(self, kind: 'OptionNumber', length: 'int') -> 'None': ...  # pylint: disable=unused-argument,super-init-not-called,multiple-statements,line-too-long,redefined-builtin
 
 
+@info_final
 class NoOperation(Option):
     """Data model for TCP no operation option."""
 
@@ -129,6 +137,7 @@ class NoOperation(Option):
         def __init__(self, kind: 'OptionNumber', length: 'int') -> 'None': ...  # pylint: disable=unused-argument,super-init-not-called,multiple-statements,line-too-long,redefined-builtin
 
 
+@info_final
 class MaximumSegmentSize(Option):
     """Data model for TCP maximum segment size option."""
 
@@ -139,6 +148,7 @@ class MaximumSegmentSize(Option):
         def __init__(self, kind: 'OptionNumber', length: 'int', mss: 'int') -> 'None': ...  # pylint: disable=unused-argument,super-init-not-called,multiple-statements,line-too-long,redefined-builtin
 
 
+@info_final
 class WindowScale(Option):
     """Data model for TCP window scale option."""
 
@@ -149,6 +159,7 @@ class WindowScale(Option):
         def __init__(self, kind: 'OptionNumber', length: 'int', shift: 'int') -> 'None': ...  # pylint: disable=unused-argument,super-init-not-called,multiple-statements,line-too-long,redefined-builtin
 
 
+@info_final
 class SACKPermitted(Option):
     """Data model for TCP SACK permitted option."""
 
@@ -156,16 +167,31 @@ class SACKPermitted(Option):
         def __init__(self, kind: 'OptionNumber', length: 'int') -> 'None': ...  # pylint: disable=unused-argument,super-init-not-called,multiple-statements,line-too-long,redefined-builtin
 
 
+@info_final
+class SACKBlock(Data):
+    """Data model for TCP SACK block."""
+
+    #: Left edge.
+    left: 'int'
+    #: Right edge.
+    right: 'int'
+
+    if TYPE_CHECKING:
+        def __init__(self, left: 'int', right: 'int') -> 'None': ...  # pylint: disable=unused-argument,super-init-not-called,multiple-statements,line-too-long,redefined-builtin
+
+
+@info_final
 class SACK(Option):
     """Data model for TCP SACK option."""
 
     #: SACK blocks.
-    sack: 'tuple[int, ...]'
+    sack: 'tuple[SACKBlock, ...]'
 
     if TYPE_CHECKING:
-        def __init__(self, kind: 'OptionNumber', length: 'int', sack: 'tuple[int, ...]') -> 'None': ...  # pylint: disable=unused-argument,super-init-not-called,multiple-statements,line-too-long,redefined-builtin
+        def __init__(self, kind: 'OptionNumber', length: 'int', sack: 'tuple[SACKBlock, ...]') -> 'None': ...  # pylint: disable=unused-argument,super-init-not-called,multiple-statements,line-too-long,redefined-builtin
 
 
+@info_final
 class Echo(Option):
     """Data model for TCP echo option."""
 
@@ -176,6 +202,7 @@ class Echo(Option):
         def __init__(self, kind: 'OptionNumber', length: 'int', data: 'bytes') -> 'None': ...  # pylint: disable=unused-argument,super-init-not-called,multiple-statements,line-too-long,redefined-builtin
 
 
+@info_final
 class EchoReply(Option):
     """Data model for TCP echo reply option."""
 
@@ -186,18 +213,20 @@ class EchoReply(Option):
         def __init__(self, kind: 'OptionNumber', length: 'int', data: 'bytes') -> 'None': ...  # pylint: disable=unused-argument,super-init-not-called,multiple-statements,line-too-long,redefined-builtin
 
 
+@info_final
 class Timestamp(Option):
     """Data model for TCP timestamp option."""
 
     #: Timestamp .
     timestamp: 'int'
     #: Echo data.
-    echo: 'bytes'
+    echo: 'int'
 
     if TYPE_CHECKING:
-        def __init__(self, kind: 'OptionNumber', length: 'int', timestamp: 'int', echo: 'bytes') -> 'None': ...  # pylint: disable=unused-argument,super-init-not-called,multiple-statements,line-too-long,redefined-builtin
+        def __init__(self, kind: 'OptionNumber', length: 'int', timestamp: 'int', echo: 'int') -> 'None': ...  # pylint: disable=unused-argument,super-init-not-called,multiple-statements,line-too-long,redefined-builtin
 
 
+@info_final
 class PartialOrderConnectionPermitted(Option):
     """Data model for TCP partial order connection permitted option."""
 
@@ -205,6 +234,7 @@ class PartialOrderConnectionPermitted(Option):
         def __init__(self, kind: 'OptionNumber', length: 'int') -> 'None': ...  # pylint: disable=unused-argument,super-init-not-called,multiple-statements,line-too-long,redefined-builtin
 
 
+@info_final
 class PartialOrderConnectionProfile(Option):
     """Data model for TCP partial order connection profile option."""
 
@@ -217,6 +247,7 @@ class PartialOrderConnectionProfile(Option):
         def __init__(self, kind: 'OptionNumber', length: 'int', start: 'bool', end: 'bool') -> 'None': ...  # pylint: disable=unused-argument,super-init-not-called,multiple-statements,line-too-long,redefined-builtin
 
 
+@info_final
 class CC(Option):
     """Data model for TCP CC option."""
 
@@ -227,6 +258,7 @@ class CC(Option):
         def __init__(self, kind: 'OptionNumber', length: 'int', cc: 'int') -> 'None': ...  # pylint: disable=unused-argument,super-init-not-called,multiple-statements,line-too-long,redefined-builtin
 
 
+@info_final
 class CCNew(Option):
     """Data model for TCP CC.NEW option."""
 
@@ -237,6 +269,7 @@ class CCNew(Option):
         def __init__(self, kind: 'OptionNumber', length: 'int', cc: 'int') -> 'None': ...  # pylint: disable=unused-argument,super-init-not-called,multiple-statements,line-too-long,redefined-builtin
 
 
+@info_final
 class CCEcho(Option):
     """Data model for TCP CC.ECHO option."""
 
@@ -247,6 +280,7 @@ class CCEcho(Option):
         def __init__(self, kind: 'OptionNumber', length: 'int', cc: 'int') -> 'None': ...  # pylint: disable=unused-argument,super-init-not-called,multiple-statements,line-too-long,redefined-builtin
 
 
+@info_final
 class AlternateChecksumRequest(Option):
     """Data model for TCP alternate checksum request option."""
 
@@ -257,6 +291,7 @@ class AlternateChecksumRequest(Option):
         def __init__(self, kind: 'OptionNumber', length: 'int', chksum: 'Checksum') -> 'None': ...  # pylint: disable=unused-argument,super-init-not-called,multiple-statements,line-too-long,redefined-builtin
 
 
+@info_final
 class AlternateChecksumData(Option):
     """Data model for TCP alternate checksum data option."""
 
@@ -267,6 +302,7 @@ class AlternateChecksumData(Option):
         def __init__(self, kind: 'OptionNumber', length: 'int', data: 'bytes') -> 'None': ...  # pylint: disable=unused-argument,super-init-not-called,multiple-statements,line-too-long,redefined-builtin
 
 
+@info_final
 class MD5Signature(Option):
     """Data model for TCP MD5 signature option."""
 
@@ -277,6 +313,7 @@ class MD5Signature(Option):
         def __init__(self, kind: 'OptionNumber', length: 'int', digest: 'bytes') -> 'None': ...  # pylint: disable=unused-argument,super-init-not-called,multiple-statements,line-too-long,redefined-builtin
 
 
+@info_final
 class QuickStartResponse(Option):
     """Data model for TCP quick start response option."""
 
@@ -284,13 +321,14 @@ class QuickStartResponse(Option):
     req_rate: 'int'
     #: TTL difference.
     ttl_diff: 'int'
-    #: QS nounce.
-    nounce: 'int'
+    #: QS nonce.
+    nonce: 'int'
 
     if TYPE_CHECKING:
-        def __init__(self, kind: 'OptionNumber', length: 'int', req_rate: 'int', ttl_diff: 'int', nounce: 'int') -> 'None': ...  # pylint: disable=unused-argument,super-init-not-called,multiple-statements,line-too-long,redefined-builtin
+        def __init__(self, kind: 'OptionNumber', length: 'int', req_rate: 'int', ttl_diff: 'int', nonce: 'int') -> 'None': ...  # pylint: disable=unused-argument,super-init-not-called,multiple-statements,line-too-long,redefined-builtin
 
 
+@info_final
 class UserTimeout(Option):
     """Data model for TCP user timeout option."""
 
@@ -301,6 +339,7 @@ class UserTimeout(Option):
         def __init__(self, kind: 'OptionNumber', length: 'int', timeout: 'timedelta') -> 'None': ...  # pylint: disable=unused-argument,super-init-not-called,multiple-statements,line-too-long,redefined-builtin
 
 
+@info_final
 class Authentication(Option):
     """Data model for TCP authentication option."""
 
@@ -315,6 +354,7 @@ class Authentication(Option):
         def __init__(self, kind: 'OptionNumber', length: 'int', key_id: 'int', next_key_id: 'int', mac: 'bytes') -> 'None': ...  # pylint: disable=unused-argument,super-init-not-called,multiple-statements,line-too-long,redefined-builtin
 
 
+@info_final
 class FastOpenCookie(Option):
     """Data model for TCP fast open cookie option."""
 
@@ -332,6 +372,7 @@ class MPTCP(Option):
     subtype: 'MPTCPOption'
 
 
+@info_final
 class MPTCPUnknown(MPTCP):
     """Data model for TCP unknown MPTCP option."""
 
@@ -342,7 +383,8 @@ class MPTCPUnknown(MPTCP):
         def __init__(self, kind: 'OptionNumber', length: 'int', subtype: 'MPTCPOption', data: 'bytes') -> 'None': ...  # pylint: disable=unused-argument,super-init-not-called,multiple-statements,line-too-long,redefined-builtin
 
 
-class MPTCPCapableFlag(Info):
+@info_final
+class MPTCPCapableFlag(Data):
     """Data model for TCP MPTCP capable option flags."""
 
     #: Checksum require flag.
@@ -356,6 +398,7 @@ class MPTCPCapableFlag(Info):
         def __init__(self, req: 'bool', ext: 'bool', hsa: 'bool') -> 'None': ...  # pylint: disable=unused-argument,super-init-not-called,multiple-statements,line-too-long,redefined-builtin
 
 
+@info_final
 class MPTCPCapable(MPTCP):
     """Data model for TCP ``MP_CAPABLE`` option."""
 
@@ -375,30 +418,31 @@ class MPTCPCapable(MPTCP):
 class MPTCPJoin(MPTCP):
     """Data model for TCP ``MP_JOIN`` option."""
 
+    #: Connection type.
+    connection: 'TCP_Flags'
 
+
+@info_final
 class MPTCPJoinSYN(MPTCPJoin):
     """Data model for TCP ``MP_JOIN-SYN`` option."""
 
-    #: Connection type.
-    connection: 'Literal["SYN"]'
     #: Backup path flag.
     backup: 'bool'
     #: Address ID.
     addr_id: 'int'
     #: Receiver's token.
-    token: 'bytes'
+    token: 'int'
     #: Sendder's random number.
-    nounce: 'int'
+    nonce: 'int'
 
     if TYPE_CHECKING:
-        def __init__(self, kind: 'OptionNumber', length: 'int', subtype: 'MPTCPOption', connection: 'Literal["SYN"]', backup: 'bool', addr_id: 'int', token: 'bytes', nounce: 'int') -> 'None': ...  # pylint: disable=unused-argument,super-init-not-called,multiple-statements,line-too-long,redefined-builtin
+        def __init__(self, kind: 'OptionNumber', length: 'int', subtype: 'MPTCPOption', connection: 'TCP_Flags', backup: 'bool', addr_id: 'int', token: 'int', nonce: 'int') -> 'None': ...  # pylint: disable=unused-argument,super-init-not-called,multiple-statements,line-too-long,redefined-builtin
 
 
+@info_final
 class MPTCPJoinSYNACK(MPTCPJoin):
     """Data model for TCP ``MP_JOIN-SYNACK`` option."""
 
-    #: Connection type.
-    connection: 'Literal["SYN/ACK"]'
     #: Backup path flag.
     backup: 'bool'
     #: Address ID.
@@ -406,47 +450,29 @@ class MPTCPJoinSYNACK(MPTCPJoin):
     #: Sender's truncated HMAC.
     hmac: 'bytes'
     #: Sendder's random number.
-    nounce: 'int'
+    nonce: 'int'
 
     if TYPE_CHECKING:
-        def __init__(self, kind: 'OptionNumber', length: 'int', subtype: 'MPTCPOption', connection: 'Literal["SYN/ACK"]', backup: 'bool', addr_id: 'int', hmac: 'bytes', nounce: 'int') -> 'None': ...  # pylint: disable=unused-argument,super-init-not-called,multiple-statements,line-too-long,redefined-builtin
+        def __init__(self, kind: 'OptionNumber', length: 'int', subtype: 'MPTCPOption', connection: 'TCP_Flags', backup: 'bool', addr_id: 'int', hmac: 'bytes', nonce: 'int') -> 'None': ...  # pylint: disable=unused-argument,super-init-not-called,multiple-statements,line-too-long,redefined-builtin
 
 
+@info_final
 class MPTCPJoinACK(MPTCPJoin):
     """Data model for TCP ``MP_JOIN-ACK`` option."""
 
-    #: Connection type.
-    connection: 'Literal["ACK"]'
+    #: HMAC value.
     hmac: 'bytes'
 
     if TYPE_CHECKING:
-        def __init__(self, kind: 'OptionNumber', length: 'int', subtype: 'MPTCPOption', connection: 'Literal["ACK"]', hmac: 'bytes') -> 'None': ...  # pylint: disable=unused-argument,super-init-not-called,multiple-statements,line-too-long,redefined-builtin
-
-
-class MPTCPDSSFlag(Info):
-    """Data model for TCP ``DSS`` option flags."""
-
-    #: ``DATA_FIN`` flag.
-    data_fin: 'bool'
-    #: Data sequence number is 8 octets (if not set, DSN is 4 octets).
-    dsn_oct: 'bool'
-    #: Data Sequence Number (DSN), Subflow Sequence Number (SSN), Data-Level
-    #: Length, and Checksum present.
-    data_pre: 'bool'
-    #: Data ACK is 8 octets (if not set, Data ACK is 4 octets).
-    ack_oct: 'bool'
-    #: Data ACK present.
-    ack_pre: 'bool'
-
-    if TYPE_CHECKING:
-        def __init__(self, data_fin: 'bool', dsn_oct: 'bool', data_pre: 'bool', ack_oct: 'bool', ack_pre: 'bool') -> 'None': ...  # pylint: disable=unused-argument,super-init-not-called,multiple-statements,line-too-long,redefined-builtin
+        def __init__(self, kind: 'OptionNumber', length: 'int', subtype: 'MPTCPOption', connection: 'TCP_Flags', hmac: 'bytes') -> 'None': ...  # pylint: disable=unused-argument,super-init-not-called,multiple-statements,line-too-long,redefined-builtin
 
 
+@info_final
 class MPTCPDSS(MPTCP):
     """Data model for TCP ``DSS`` option."""
 
-    #: Flags.
-    flags: 'MPTCPDSSFlag'
+    #: ``DATA_FIN`` flag.
+    data_fin: 'bool'
     #: Data ACK.
     ack: 'Optional[int]'
     #: Data sequence number.
@@ -459,9 +485,10 @@ class MPTCPDSS(MPTCP):
     checksum: 'Optional[bytes]'
 
     if TYPE_CHECKING:
-        def __init__(self, kind: 'OptionNumber', length: 'int', subtype: 'MPTCPOption', flags: 'MPTCPDSSFlag', ack: 'Optional[int]', dsn: 'Optional[int]', ssn: 'Optional[int]', dl_len: 'Optional[int]', checksum: 'Optional[bytes]') -> 'None': ...  # pylint: disable=unused-argument,super-init-not-called,multiple-statements,line-too-long,redefined-builtin
+        def __init__(self, kind: 'OptionNumber', length: 'int', subtype: 'MPTCPOption', data_fin: 'bool', ack: 'Optional[int]', dsn: 'Optional[int]', ssn: 'Optional[int]', dl_len: 'Optional[int]', checksum: 'Optional[bytes]') -> 'None': ...  # pylint: disable=unused-argument,super-init-not-called,multiple-statements,line-too-long,redefined-builtin
 
 
+@info_final
 class MPTCPAddAddress(MPTCP):
     """Data model for TCP ``ADD_ADDR`` option."""
 
@@ -478,6 +505,7 @@ class MPTCPAddAddress(MPTCP):
         def __init__(self, kind: 'OptionNumber', length: 'int', subtype: 'MPTCPOption', version: 'int', addr_id: 'int', addr: 'IPAddress', port: 'Optional[int]') -> 'None': ...  # pylint: disable=unused-argument,super-init-not-called,multiple-statements,line-too-long,redefined-builtin
 
 
+@info_final
 class MPTCPRemoveAddress(MPTCP):
     """Data model for TCP ``REMOVE_ADDR`` option."""
 
@@ -488,6 +516,7 @@ class MPTCPRemoveAddress(MPTCP):
         def __init__(self, kind: 'OptionNumber', length: 'int', subtype: 'MPTCPOption', addr_id: 'tuple[int, ...]') -> 'None': ...  # pylint: disable=unused-argument,super-init-not-called,multiple-statements,line-too-long,redefined-builtin
 
 
+@info_final
 class MPTCPPriority(MPTCP):
     """Data model for TCP ``MP_PRIO`` option."""
 
@@ -500,6 +529,7 @@ class MPTCPPriority(MPTCP):
         def __init__(self, kind: 'OptionNumber', length: 'int', subtype: 'MPTCPOption', backup: 'bool', addr_id: 'Optional[int]') -> 'None': ...  # pylint: disable=unused-argument,super-init-not-called,multiple-statements,line-too-long,redefined-builtin
 
 
+@info_final
 class MPTCPFallback(MPTCP):
     """Data model for TCP ``MP_FAIL`` option."""
 
@@ -510,11 +540,12 @@ class MPTCPFallback(MPTCP):
         def __init__(self, kind: 'OptionNumber', length: 'int', subtype: 'MPTCPOption', dsn: 'int') -> 'None': ...  # pylint: disable=unused-argument,super-init-not-called,multiple-statements,line-too-long,redefined-builtin
 
 
+@info_final
 class MPTCPFastclose(MPTCP):
     """Data model for TCP ``MP_FASTCLOSE`` option."""
 
     #: Option receiver's key.
-    rkey: 'bytes'
+    rkey: 'int'
 
     if TYPE_CHECKING:
-        def __init__(self, kind: 'OptionNumber', length: 'int', subtype: 'MPTCPOption', rkey: 'bytes') -> 'None': ...  # pylint: disable=unused-argument,super-init-not-called,multiple-statements,line-too-long,redefined-builtin
+        def __init__(self, kind: 'OptionNumber', length: 'int', subtype: 'MPTCPOption', rkey: 'int') -> 'None': ...  # pylint: disable=unused-argument,super-init-not-called,multiple-statements,line-too-long,redefined-builtin
