@@ -3,12 +3,14 @@
 
 from typing import TYPE_CHECKING
 
-from pcapkit.corekit.infoclass import Info
+from pcapkit.corekit.infoclass import info_final
+from pcapkit.protocols.data.data import Data
 
 __all__ = ['UDP']
 
 
-class UDP(Info):
+@info_final
+class UDP(Data):
     """Data model for UDP protocol."""
 
     #: Source port.
