@@ -2,6 +2,8 @@
 """AH - Authentication Header
 ================================
 
+.. module:: pcapkit.protocols.internet.ah
+
 :mod:`pcapkit.protocols.internet.ah` contains
 :class:`~pcapkit.protocols.internet.AH` only,
 which implements extractor for Authentication
@@ -24,23 +26,28 @@ Octets      Bits        Name                    Description
 """
 from typing import TYPE_CHECKING, overload
 
-from pcapkit.const.reg.transtype import TransType as RegType_TransType
-from pcapkit.protocols.data.internet.ah import AH as DataType_AH
+from pcapkit.const.reg.transtype import TransType as Enum_TransType
+from pcapkit.protocols.data.internet.ah import AH as Data_AH
 from pcapkit.protocols.internet.ipsec import IPsec
-from pcapkit.utilities.exceptions import ProtocolError, UnsupportedCall, VersionError
+from pcapkit.protocols.schema.internet.ah import AH as Schema_AH
+from pcapkit.utilities.exceptions import UnsupportedCall
 
 if TYPE_CHECKING:
-    from typing import Any, BinaryIO, NoReturn, Optional
+    from enum import IntEnum as StdlibEnum
+    from typing import IO, Any, NoReturn, Optional, Type
 
+    from aenum import IntEnum as AenumEnum
     from typing_extensions import Literal
 
     from pcapkit.corekit.protochain import ProtoChain
     from pcapkit.protocols.protocol import Protocol
+    from pcapkit.protocols.schema.schema import Schema
 
 __all__ = ['AH']
 
 
-class AH(IPsec[DataType_AH]):
+class AH(IPsec[Data_AH, Schema_AH],
+         schema=Schema_AH, data=Data_AH):
     """This class implements Authentication Header."""
 
     ##########################################################################
@@ -98,7 +105,7 @@ class AH(IPsec[DataType_AH]):
     ##########################################################################
 
     def read(self, length: 'Optional[int]' = None, *, version: 'Literal[4, 6]' = 4,  # pylint: disable=arguments-differ
-             extension: bool = False, **kwargs: 'Any') -> 'DataType_AH':  # pylint: disable=unused-argument
+             extension: bool = False, **kwargs: 'Any') -> 'Data_AH':  # pylint: disable=unused-argument
         """Read Authentication Header.
 
         Structure of AH header [:rfc:`4302`]:
@@ -131,53 +138,59 @@ class AH(IPsec[DataType_AH]):
         """
         if length is None:
             length = len(self)
+        schema = self.__header__
 
-        _next = self._read_protos(1)
-        _plen = self._read_unpack(1)
-        _resv = self._read_fileng(2)
-        _scpi = self._read_unpack(4)
-        _dsnf = self._read_unpack(4)
-
-        # ICV length & value
-        _tlen = _plen * 4 - 2
-        _vlen = _tlen - 12
-        _chkv = self._read_fileng(_vlen)
-
-        ah = DataType_AH(
-            next=_next,
-            length=_tlen,
-            spi=_scpi,
-            seq=_dsnf,
-            icv=_chkv,
+        ah = Data_AH(
+            next=schema.next,
+            length=(schema.len + 2) * 4,
+            spi=schema.spi,
+            seq=schema.seq,
+            icv=schema.icv,
         )
 
-        if version == 6:
-            _plen = 8 - (_tlen % 8)
-        elif version == 4:
-            _plen = 4 - (_tlen % 4)
-        else:
-            raise VersionError(f'Unknown IP version {version}')
-
-        if _plen:   # explicit padding in need
-            padding = self._read_binary(_plen)
-            if int(padding, base=2) != 0:  # check padding (all zero)
-                raise ProtocolError(f'{self.alias}: invalid format')
-
         if extension:
             return ah
-        return self._decode_next_layer(ah, _next, length - ah.length)
+        return self._decode_next_layer(ah, schema.next, length - ah.length)
 
-    def make(self, **kwargs: 'Any') -> 'NoReturn':
+    def make(self,
+             next: 'Enum_TransType | StdlibEnum | AenumEnum | str | int' = Enum_TransType.UDP,
+             next_default: 'Optional[int]' = None,
+             next_namespace: 'Optional[dict[str, int] | dict[int, str] | Type[StdlibEnum] | Type[AenumEnum]]' = None,  # pylint: disable=line-too-long
+             next_reversed: 'bool' = False,
+             spi: 'int' = 0,
+             seq: 'int' = 0,
+             icv: 'bytes' = b'',
+             payload: 'bytes | Protocol | Schema' = b'',
+             **kwargs: 'Any') -> 'Schema_AH':
         """Make (construct) packet data.
 
         Args:
+            next: Next header type.
+            next_default: Default value of next header type.
+            next_namespace: Namespace of next header type.
+            next_reversed: If the namespace is reversed.
+            spi: Security Parameters Index.
+            seq: Sequence Number Field.
+            icv: Integrity Check Value-ICV.
+            payload: Payload of current instance.
             **kwargs: Arbitrary keyword arguments.
 
         Returns:
             Constructed packet data.
 
         """
-        raise NotImplementedError
+        next_value = self._make_index(next, next_default, namespace=next_namespace,
+                                      reversed=next_reversed, pack=False)
+        length = (len(icv) + 12) // 4 - 2
+
+        return Schema_AH(
+            next=next_value,  # type: ignore[arg-type]
+            len=length,
+            spi=spi,
+            seq=seq,
+            icv=icv,
+            payload=payload,
+        )
 
     @classmethod
     def id(cls) -> 'tuple[Literal["AH"]]':  # type: ignore[override]
@@ -194,13 +207,13 @@ class AH(IPsec[DataType_AH]):
     ##########################################################################
 
     @overload
-    def __post_init__(self, file: 'BinaryIO', length: 'Optional[int]' = ..., *,  # pylint: disable=arguments-differ
+    def __post_init__(self, file: 'IO[bytes] | bytes', length: 'Optional[int]' = ..., *,  # pylint: disable=arguments-differ
                       version: 'Literal[4, 6]' = ..., extension: 'bool' = ...,
                       **kwargs: 'Any') -> 'None': ...
     @overload
     def __post_init__(self, **kwargs: 'Any') -> 'None': ...  # pylint: disable=arguments-differ
 
-    def __post_init__(self, file: 'Optional[BinaryIO]' = None, length: 'Optional[int]' = None, *,  # pylint: disable=arguments-differ
+    def __post_init__(self, file: 'Optional[IO[bytes] | bytes]' = None, length: 'Optional[int]' = None, *,  # pylint: disable=arguments-differ
                       version: 'Literal[4, 6]' = 4, extension: 'bool' = False,
                       **kwargs: 'Any') -> 'None':
         """Post initialisation hook.
@@ -227,7 +240,7 @@ class AH(IPsec[DataType_AH]):
         return 20
 
     @classmethod
-    def __index__(cls) -> 'RegType_TransType':  # pylint: disable=invalid-index-returned
+    def __index__(cls) -> 'Enum_TransType':  # pylint: disable=invalid-index-returned
         """Numeral registry index of the protocol.
 
         Returns:
@@ -236,4 +249,27 @@ class AH(IPsec[DataType_AH]):
         .. _IANA: https://www.iana.org/assignments/protocol-numbers/protocol-numbers.xhtml
 
         """
-        return RegType_TransType.AH  # type: ignore[return-value]
+        return Enum_TransType.AH  # type: ignore[return-value]
+
+    ##########################################################################
+    # Utilities.
+    ##########################################################################
+
+    @classmethod
+    def _make_data(cls, data: 'Data_AH') -> 'dict[str, Any]':  # type: ignore[override]
+        """Create key-value pairs from ``data`` for protocol construction.
+
+        Args:
+            data: protocol data
+
+        Returns:
+            Key-value pairs for protocol construction.
+
+        """
+        return {
+            'next': data.next,
+            'spi': data.spi,
+            'seq': data.seq,
+            'icv': data.icv,
+            'payload': cls._make_payload(data),
+        }
