@@ -2,6 +2,8 @@
 """Base Protocol
 ===================
 
+.. module:: pcapkit.protocols.internet.internet
+
 :mod:`pcapkit.protocols.internet.internet` contains :class:`~pcapkit.protocols.internet.internet.Internet`,
 which is a base class for internet layer protocols, eg. :class:`~pcapkit.protocols.internet.ah.AH`,
 :class:`~pcapkit.protocols.internet.ipsec.IPsec`, :class:`~pcapkit.protocols.internet.ipv4.IPv4`,
@@ -12,20 +14,21 @@ import collections
 import importlib
 from typing import TYPE_CHECKING, Generic, cast
 
-from pcapkit.const.reg.transtype import TransType as RegType_TransType
+from pcapkit.const.reg.transtype import TransType as Enum_TransType
 from pcapkit.corekit.protochain import ProtoChain
-from pcapkit.protocols.protocol import PT, Protocol
+from pcapkit.protocols.protocol import PT, ST, Protocol
 from pcapkit.utilities.decorators import beholder
+from pcapkit.utilities.warnings import RegistryWarning, warn
 
 if TYPE_CHECKING:
-    from typing import Optional, Type
+    from typing import Any, Optional, Type
 
     from typing_extensions import Literal
 
 __all__ = ['Internet']
 
 
-class Internet(Protocol[PT], Generic[PT]):  # pylint: disable=abstract-method
+class Internet(Protocol[PT, ST], Generic[PT, ST]):  # pylint: disable=abstract-method
     """Abstract base class for internet layer protocol family.
 
     This class currently supports parsing of the following protocols, which are
@@ -79,19 +82,19 @@ class Internet(Protocol[PT], Generic[PT]):  # pylint: disable=abstract-method
     __proto__ = collections.defaultdict(
         lambda: ('pcapkit.protocols.misc.raw', 'Raw'),
         {
-            RegType_TransType.HOPOPT:          ('pcapkit.protocols.internet.hopopt',     'HOPOPT'),
-            RegType_TransType.IPv4:            ('pcapkit.protocols.internet.ipv4',       'IPv4'),
-            RegType_TransType.TCP:             ('pcapkit.protocols.transport.tcp',       'TCP'),
-            RegType_TransType.UDP:             ('pcapkit.protocols.transport.udp',       'UDP'),
-            RegType_TransType.IPv6:            ('pcapkit.protocols.internet.ipv6',       'IPv6'),
-            RegType_TransType.IPv6_Route:      ('pcapkit.protocols.internet.ipv6_route', 'IPv6_Route'),
-            RegType_TransType.IPv6_Frag:       ('pcapkit.protocols.internet.ipv6_frag',  'IPv6_Frag'),
-            RegType_TransType.AH:              ('pcapkit.protocols.internet.ah',         'AH'),
-            RegType_TransType.IPv6_NoNxt:      ('pcapkit.protocols.misc.raw',            'Raw'),
-            RegType_TransType.IPv6_Opts:       ('pcapkit.protocols.internet.ipv6_opts',  'IPv6_Opts'),
-            RegType_TransType.IPX_in_IP:       ('pcapkit.protocols.internet.ipx',        'IPX'),
-            RegType_TransType.Mobility_Header: ('pcapkit.protocols.internet.mh',         'MH'),
-            RegType_TransType.HIP:             ('pcapkit.protocols.internet.hip',        'HIP'),
+            Enum_TransType.HOPOPT:          ('pcapkit.protocols.internet.hopopt',     'HOPOPT'),
+            Enum_TransType.IPv4:            ('pcapkit.protocols.internet.ipv4',       'IPv4'),
+            Enum_TransType.TCP:             ('pcapkit.protocols.transport.tcp',       'TCP'),
+            Enum_TransType.UDP:             ('pcapkit.protocols.transport.udp',       'UDP'),
+            Enum_TransType.IPv6:            ('pcapkit.protocols.internet.ipv6',       'IPv6'),
+            Enum_TransType.IPv6_Route:      ('pcapkit.protocols.internet.ipv6_route', 'IPv6_Route'),
+            Enum_TransType.IPv6_Frag:       ('pcapkit.protocols.internet.ipv6_frag',  'IPv6_Frag'),
+            Enum_TransType.AH:              ('pcapkit.protocols.internet.ah',         'AH'),
+            Enum_TransType.IPv6_NoNxt:      ('pcapkit.protocols.misc.raw',            'Raw'),
+            Enum_TransType.IPv6_Opts:       ('pcapkit.protocols.internet.ipv6_opts',  'IPv6_Opts'),
+            Enum_TransType.IPX_in_IP:       ('pcapkit.protocols.internet.ipx',        'IPX'),
+            Enum_TransType.Mobility_Header: ('pcapkit.protocols.internet.mh',         'MH'),
+            Enum_TransType.HIP:             ('pcapkit.protocols.internet.hip',        'HIP'),
         },
     )
 
@@ -110,7 +113,7 @@ class Internet(Protocol[PT], Generic[PT]):  # pylint: disable=abstract-method
     ##########################################################################
 
     @classmethod
-    def register(cls, code: 'RegType_TransType', module: str, class_: str) -> 'None':
+    def register(cls, code: 'Enum_TransType', module: str, class_: str) -> 'None':  # type: ignore[override]
         r"""Register a new protocol class.
 
         Notes:
@@ -123,13 +126,15 @@ class Internet(Protocol[PT], Generic[PT]):  # pylint: disable=abstract-method
             class\_: class name
 
         """
+        if code in cls.__proto__:
+            warn(f'protocol {code} already registered, overwriting', RegistryWarning)
         cls.__proto__[code] = (module, class_)
 
     ##########################################################################
     # Utilities.
     ##########################################################################
 
-    def _read_protos(self, size: 'int') -> 'RegType_TransType':
+    def _read_protos(self, size: 'int') -> 'Enum_TransType':
         """Read next layer protocol type.
 
         Arguments:
@@ -140,26 +145,34 @@ class Internet(Protocol[PT], Generic[PT]):  # pylint: disable=abstract-method
 
         """
         _byte = self._read_unpack(size)
-        _prot = RegType_TransType.get(_byte)
+        _prot = Enum_TransType.get(_byte)
         return _prot
 
     def _decode_next_layer(self, dict_: 'PT', proto: 'Optional[int]' = None,  # pylint: disable=arguments-differ
-                           length: 'Optional[int]' = None, *, version: 'Literal[4, 6]' = 4,
-                           ipv6_exthdr: 'Optional[ProtoChain]' = None) -> 'PT':
+                           length: 'Optional[int]' = None, *, packet: 'Optional[dict[str, Any]]' = None,
+                           version: 'Literal[4, 6]' = 4, ipv6_exthdr: 'Optional[ProtoChain]' = None) -> 'PT':
         r"""Decode next layer extractor.
 
         Arguments:
             dict\_: info buffer
             proto: next layer protocol index
             length: valid (*non-padding*) length
+            packet: packet info (passed from :meth:`self.unpack <pcapkit.protocols.protocol.Protocol.unpack>`)
             version: IP version
             ipv6_exthdr: protocol chain of IPv6 extension headers
 
         Returns:
             Current protocol with next layer extracted.
 
+        Notes:
+            We added a new key ``__next_type__`` to ``dict_`` to store the
+            next layer protocol type, and a new key ``__next_name__`` to
+            store the next layer protocol name. These two keys will **NOT**
+            be included when :meth:`Info.to_dict <pcapkit.corekit.infoclass.Info.to_dict>` is called.
+
         """
-        next_ = self._import_next_layer(proto, length, version=version)  # type: ignore[misc,call-arg,arg-type]
+        next_ = cast('Protocol',  # type: ignore[redundant-cast]
+                     self._import_next_layer(proto, length, packet=packet, version=version))  # type: ignore[arg-type,misc,call-arg]
         info, chain = next_.info, next_.protochain
 
         # make next layer protocol name
@@ -167,7 +180,11 @@ class Internet(Protocol[PT], Generic[PT]):  # pylint: disable=abstract-method
         # proto = next_.__class__.__name__
 
         # write info and protocol chain into dict
-        dict_.__update__([(layer, info)])
+        dict_.__update__({
+            layer: info,
+            '__next_type__': type(next_),
+            '__next_name__': layer,
+        })
         self._next = next_  # pylint: disable=attribute-defined-outside-init
         if ipv6_exthdr is not None:
             if chain is not None:
@@ -179,12 +196,14 @@ class Internet(Protocol[PT], Generic[PT]):  # pylint: disable=abstract-method
 
     @beholder  # type: ignore[arg-type]
     def _import_next_layer(self, proto: 'int', length: 'Optional[int]' = None, *,  # pylint: disable=arguments-differ
+                           packet: 'Optional[dict[str, Any]]' = None,
                            version: 'Literal[4, 6]' = 4, extension: 'bool' = False) -> 'Protocol':
         """Import next layer extractor.
 
         Arguments:
             proto: next layer protocol index
             length: valid (*non-padding*) length
+            packet: packet info (passed from :meth:`self.unpack <pcapkit.protocols.protocol.Protocol.unpack>`)
             version: IP protocol version
             extension: if is extension header
 
@@ -195,14 +214,18 @@ class Internet(Protocol[PT], Generic[PT]):  # pylint: disable=abstract-method
         if TYPE_CHECKING:
             protocol: 'Type[Protocol]'
 
-        if length is not None and length == 0:
-            from pcapkit.protocols.misc.null import NoPayload as protocol  # type: ignore[no-redef] # isort: skip # pylint: disable=import-outside-toplevel
+        file_ = self.__header__.get_payload()
+        if length is None:
+            length = len(file_)
+
+        if length == 0:
+            from pcapkit.protocols.misc.null import NoPayload as protocol  # isort: skip # pylint: disable=import-outside-toplevel
         elif self._sigterm:
-            from pcapkit.protocols.misc.raw import Raw as protocol  # type: ignore[no-redef] # isort: skip # pylint: disable=import-outside-toplevel
+            from pcapkit.protocols.misc.raw import Raw as protocol  # isort: skip # pylint: disable=import-outside-toplevel
         else:
             module, name = self.__proto__[proto]
             protocol = cast('Type[Protocol]', getattr(importlib.import_module(module), name))
 
-        next_ = protocol(self._file, length, version=version, extension=extension,  # type: ignore[abstract]
-                         alias=proto, layer=self._exlayer, protocol=self._exproto)
+        next_ = protocol(file_, length, version=version, extension=extension,  # type: ignore[abstract]
+                         alias=proto, packet=packet, layer=self._exlayer, protocol=self._exproto)
         return next_
