@@ -2,6 +2,8 @@
 """IPX - Internetwork Packet Exchange
 ========================================
 
+.. module:: pcapkit.protocols.internet.ipx
+
 :mod:`pcapkit.protocols.internet.ipx` contains
 :class:`~pcapkit.protocols.internet.ipx.IPX` only,
 which implements extractor for Internetwork Packet
@@ -25,22 +27,29 @@ Octets      Bits        Name                    Description
 import textwrap
 from typing import TYPE_CHECKING
 
-from pcapkit.const.ipx.packet import Packet as RegType_Packet
-from pcapkit.const.ipx.socket import Socket as RegType_Socket
-from pcapkit.const.reg.transtype import TransType as RegType_TransType
-from pcapkit.protocols.data.internet.ipx import IPX as DataType_IPX
-from pcapkit.protocols.data.internet.ipx import Address as DataType_Address
+from pcapkit.const.ipx.packet import Packet as Enum_Packet
+from pcapkit.const.ipx.socket import Socket as Enum_Socket
+from pcapkit.const.reg.transtype import TransType as Enum_TransType
+from pcapkit.protocols.data.internet.ipx import IPX as Data_IPX
+from pcapkit.protocols.data.internet.ipx import Address as Data_Address
 from pcapkit.protocols.internet.internet import Internet
+from pcapkit.protocols.schema.internet.ipx import IPX as Schema_IPX
 
 if TYPE_CHECKING:
-    from typing import Any, NoReturn, Optional
+    from enum import IntEnum as StdlibEnum
+    from typing import Any, Optional, Type
 
+    from aenum import IntEnum as AenumEnum
     from typing_extensions import Literal
 
+    from pcapkit.protocols.protocol import Protocol
+    from pcapkit.protocols.schema.schema import Schema
+
 __all__ = ['IPX']
 
 
-class IPX(Internet[DataType_IPX]):
+class IPX(Internet[Data_IPX, Schema_IPX],
+          schema=Schema_IPX, data=Data_IPX):
     """This class implements Internetwork Packet Exchange."""
 
     ##########################################################################
@@ -58,7 +67,7 @@ class IPX(Internet[DataType_IPX]):
         return 30
 
     @property
-    def protocol(self) -> 'RegType_TransType':
+    def protocol(self) -> 'Enum_TransType':
         """Name of next layer protocol."""
         return self._info.type
 
@@ -76,49 +85,73 @@ class IPX(Internet[DataType_IPX]):
     # Methods.
     ##########################################################################
 
-    def read(self, length: 'Optional[int]' = None, **kwargs: 'Any') -> 'DataType_IPX':
+    def read(self, length: 'Optional[int]' = None, **kwargs: 'Any') -> 'Data_IPX':
         """Read Internetwork Packet Exchange.
 
-         Args:
+        Args:
             length: Length of packet data.
             **kwargs: Arbitrary keyword arguments.
 
         Returns:
-            DataType_IPX: Parsed packet data.
+            Parsed packet data.
 
         """
         if length is None:
             length = len(self)
+        schema = self.__header__
 
-        _csum = self._read_fileng(2)
-        _tlen = self._read_unpack(2)
-        _ctrl = self._read_unpack(1)
-        _type = self._read_unpack(1)
-        _dsta = self._read_ipx_address()
-        _srca = self._read_ipx_address()
-
-        ipx = DataType_IPX(
-            chksum=_csum,
-            len=_tlen,
-            count=_ctrl,
-            type=RegType_Packet.get(_type),
-            dst=_dsta,
-            src=_srca,
+        ipx = Data_IPX(
+            chksum=schema.chksum,
+            len=schema.len,
+            count=schema.count,
+            type=schema.type,
+            dst=self._read_ipx_address(schema.dst),
+            src=self._read_ipx_address(schema.src),
         )
 
         return self._decode_next_layer(ipx, ipx.type, ipx.len - 30)
 
-    def make(self, **kwargs: 'Any') -> 'NoReturn':
+    def make(self,
+             chksum: 'bytes' = b'\x00\x00',
+             count: 'int' = 0,
+             type: 'Enum_Packet | StdlibEnum | AenumEnum | str | int' = Enum_Packet.Unknown,
+             type_default: 'Optional[int]' = None,
+             type_namespace: 'Optional[dict[str, int] | dict[int, str] | Type[StdlibEnum] | Type[AenumEnum]]' = None,  # pylint: disable=line-too-long
+             type_reversed: 'bool' = False,
+             dst: 'bytes' = b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00',
+             src: 'bytes' = b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00',
+             payload: 'bytes | Protocol | Schema' = b'',
+             **kwargs: 'Any') -> 'Schema_IPX':
         """Make (construct) packet data.
 
         Args:
+            chksum: Checksum.
+            count: Transport Control (hop count).
+            type: Packet Type.
+            type_default: Default value for undefined packet type.
+            type_namespace: Namespace for packet type.
+            type_reversed: Reverse namespace for packet type.
+            dst: Destination Address.
+            src: Source Address.
+            payload: Payload data.
             **kwargs: Arbitrary keyword arguments.
 
         Returns:
             bytes: Constructed packet data.
 
         """
-        raise NotImplementedError
+        type_val = self._make_index(type, type_default, namespace=type_namespace,
+                                    reversed=type_reversed, pack=False)
+
+        return Schema_IPX(
+            chksum=chksum,
+            len=30 + len(payload),
+            count=count,
+            type=type_val,  # type: ignore[arg-type]
+            dst=dst,
+            src=src,
+            payload=payload,
+        )
 
     ##########################################################################
     # Data models.
@@ -129,7 +162,7 @@ class IPX(Internet[DataType_IPX]):
         return 30
 
     @classmethod
-    def __index__(cls) -> 'RegType_TransType':  # pylint: disable=invalid-index-returned
+    def __index__(cls) -> 'Enum_TransType':  # pylint: disable=invalid-index-returned
         """Numeral registry index of the protocol.
 
         Returns:
@@ -138,40 +171,60 @@ class IPX(Internet[DataType_IPX]):
         .. _IANA: https://www.iana.org/assignments/protocol-numbers/protocol-numbers.xhtml
 
         """
-        return RegType_TransType.IPX_in_IP  # type: ignore[return-value]
+        return Enum_TransType.IPX_in_IP  # type: ignore[return-value]
 
     ##########################################################################
     # Utilities.
     ##########################################################################
 
-    def _read_ipx_address(self) -> 'DataType_Address':
+    @classmethod
+    def _make_data(cls, data: 'Data_IPX') -> 'dict[str, Any]':  # type: ignore[override]
+        """Create key-value pairs from ``data`` for protocol construction.
+
+        Args:
+            data: protocol data
+
+        Returns:
+            Key-value pairs for protocol construction.
+
+        """
+        return {
+            'chksum': data.chksum,
+            'count': data.count,
+            'type': data.type,
+            'dst': data.dst,
+            'src': data.src,
+            'payload': cls._make_payload(data),
+        }
+
+    def _read_ipx_address(self, addr: 'bytes') -> 'Data_Address':
         """Read IPX address field.
 
+        Args:
+            addr: IPX address data.
+
         Returns:
             Parsed IPX address field.
 
         """
         # Address Number
-        _byte = self._read_fileng(4)
-        _ntwk = ':'.join(textwrap.wrap(_byte.hex(), 2))
+        _ntwk = ':'.join(textwrap.wrap(addr[:4].hex(), 2))
 
         # Node Number (MAC)
-        _byte = self._read_fileng(6)
-        _node = ':'.join(textwrap.wrap(_byte.hex(), 2))
-        _maca = '-'.join(textwrap.wrap(_byte.hex(), 2))
+        _node = ':'.join(textwrap.wrap(addr[4:10].hex(), 2))
+        _maca = '-'.join(textwrap.wrap(addr[4:10].hex(), 2))
 
         # Socket Number
-        _sock = self._read_fileng(2)
+        _sock = addr[10:12]
 
         # Whole Address
-        _list = [_ntwk, _node, _sock.hex()]
+        _list = [_ntwk, _node, ':'.join(textwrap.wrap(_sock.hex(), 2))]
         _addr = ':'.join(_list)
 
-        addr = DataType_Address(
+        ipx_addr = Data_Address(
             network=_ntwk,
             node=_maca,
-            socket=RegType_Socket.get(int(_sock.hex(), base=16)),
+            socket=Enum_Socket.get(int.from_bytes(_sock, 'big', signed=False)),
             addr=_addr,
         )
-
-        return addr
+        return ipx_addr
