@@ -2,6 +2,8 @@
 """No-Payload Packet
 =======================
 
+.. module:: pcapkit.protocols.misc.null
+
 :mod:`pcapkit.protocols.null` contains
 :class:`~pcapkit.protocols.null.NoPayload` only, which
 implements a :class:`~pcapkit.protocols.protocol.Protocol` like
@@ -12,19 +14,21 @@ object whose payload is recursively
 import io
 from typing import TYPE_CHECKING, overload
 
-from pcapkit.protocols.data.misc.null import NoPayload as DataType_NoPayload
+from pcapkit.protocols.data.misc.null import NoPayload as Data_NoPayload
 from pcapkit.protocols.protocol import Protocol
+from pcapkit.protocols.schema.misc.null import NoPayload as Schema_NoPayload
 from pcapkit.utilities.exceptions import UnsupportedCall
 
 if TYPE_CHECKING:
-    from typing import Any, BinaryIO, NoReturn, Optional
+    from typing import IO, Any, NoReturn, Optional
 
     from typing_extensions import Literal
 
 __all__ = ['NoPayload']
 
 
-class NoPayload(Protocol[DataType_NoPayload]):
+class NoPayload(Protocol[Data_NoPayload, Schema_NoPayload],
+                schema=Schema_NoPayload, data=Data_NoPayload):
     """This class implements no-payload protocol."""
 
     ##########################################################################
@@ -58,7 +62,7 @@ class NoPayload(Protocol[DataType_NoPayload]):
     # Methods.
     ##########################################################################
 
-    def read(self, length: 'Optional[int]' = None, **kwargs: 'Any') -> 'DataType_NoPayload':  # pylint: disable=unused-argument
+    def read(self, length: 'Optional[int]' = None, **kwargs: 'Any') -> 'Data_NoPayload':  # pylint: disable=unused-argument
         """Read (parse) packet data.
 
         Args:
@@ -69,30 +73,30 @@ class NoPayload(Protocol[DataType_NoPayload]):
             Parsed packet data.
 
         """
-        return DataType_NoPayload()
+        return Data_NoPayload()
 
-    def make(self, **kwargs: 'Any') -> 'bytes':
+    def make(self, **kwargs: 'Any') -> 'Schema_NoPayload':
         """Make (construct) packet data.
 
         Args:
             **kwargs: Arbitrary keyword arguments.
 
         Returns:
-            Constructed packet data.
+            Constructed packet schema.
 
         """
-        return b''
+        return Schema_NoPayload()
 
     ##########################################################################
     # Data models.
     ##########################################################################
 
     @overload
-    def __post_init__(self, file: 'BinaryIO', length: 'Optional[int]' = ..., **kwargs: 'Any') -> 'None': ...
+    def __post_init__(self, file: 'IO[bytes] | bytes', length: 'Optional[int]' = ..., **kwargs: 'Any') -> 'None': ...
     @overload
     def __post_init__(self, **kwargs: 'Any') -> 'None': ...  # pylint: disable=arguments-differ
 
-    def __post_init__(self, file: 'Optional[BinaryIO]' = None,  # pylint: disable=unused-argument
+    def __post_init__(self, file: 'Optional[IO[bytes] | bytes]' = None,  # pylint: disable=unused-argument
                       length: 'Optional[int]' = None, **kwargs: 'Any') -> 'None':
         """Post initialisation hook.
 
@@ -103,11 +107,11 @@ class NoPayload(Protocol[DataType_NoPayload]):
 
         """
         #: bytes: Raw packet data.
-        self._data = bytes()
+        self._data = b''
         #: io.BytesIO: Source data stream.
         self._file = io.BytesIO()
         #: pcapkit.protocols.data.misc.null.NoPayload: Info dict of current instance.
-        self._info = self.read(length, **kwargs)
+        self._info = Data_NoPayload()
 
         #: pcapkit.protocols.null.NoPayload: Payload of current instance.
         self._next = self
