@@ -2,6 +2,8 @@
 """Frame Header
 ==================
 
+.. module:: pcapkit.protocols.misc.pcap.frame
+
 :mod:`pcapkit.protocols.misc.pcap.frame` contains
 :class:`~pcapkit.protocols.misc.pcap.frame.Frame` only,
 which implements extractor for frame headers [*]_ of PCAP,
@@ -25,39 +27,35 @@ import decimal
 import io
 import sys
 import time
-from typing import TYPE_CHECKING, overload
+from typing import TYPE_CHECKING, cast, overload
 
-from pcapkit.const.reg.linktype import LinkType as RegType_LinkType
-from pcapkit.protocols.data.misc.pcap.frame import Frame as DataType_Frame
-from pcapkit.protocols.data.misc.pcap.frame import FrameInfo as DataType_FrameInfo
+from pcapkit.const.reg.linktype import LinkType as Enum_LinkType
+from pcapkit.protocols.data.misc.pcap.frame import Frame as Data_Frame
+from pcapkit.protocols.data.misc.pcap.frame import FrameInfo as Data_FrameInfo
 from pcapkit.protocols.protocol import Protocol
-from pcapkit.utilities.exceptions import StructError, UnsupportedCall
+from pcapkit.protocols.schema.misc.pcap.frame import Frame as Schema_Frame
+from pcapkit.utilities.compat import localcontext
+from pcapkit.utilities.exceptions import UnsupportedCall, stacklevel
+from pcapkit.utilities.warnings import ProtocolWarning, RegistryWarning, warn
 
 if TYPE_CHECKING:
+    from datetime import datetime as dt_type
     from decimal import Decimal
-    from typing import Any, BinaryIO, Optional, Type
+    from typing import IO, Any, DefaultDict, Optional, Type
 
     from typing_extensions import Literal
 
-    from pcapkit.protocols.data.misc.pcap.header import Header as DataType_Header
+    from pcapkit.protocols.data.misc.pcap.header import Header as Data_Header
+    from pcapkit.protocols.schema.schema import Schema
 
 __all__ = ['Frame']
 
-if False:
-    version_info = NotImplemented
-
-
-def _walrus_wrapper_version_info_b968852604704656ae82a91cd145bb1e(expr):
-    """Wrapper function for assignment expression."""
-    global version_info
-    version_info = expr
-    return version_info
-
 # check Python version
-py37 = ((_walrus_wrapper_version_info_b968852604704656ae82a91cd145bb1e(sys.version_info)).major >= 3 and version_info.minor >= 7)
+py37 = ((version_info := sys.version_info).major >= 3 and version_info.minor >= 7)
 
 
-class Frame(Protocol[DataType_Frame]):
+class Frame(Protocol[Data_Frame, Schema_Frame],
+            schema=Schema_Frame, data=Data_Frame):
     """Per packet frame header extractor.
 
     This class currently supports parsing of the following protocols, which are
@@ -82,18 +80,18 @@ class Frame(Protocol[DataType_Frame]):
     # Defaults.
     ##########################################################################
 
-    #: DefaultDict[int, tuple[str, str]]: Protocol index mapping for decoding next layer,
-    #: c.f. :meth:`self._decode_next_layer <pcapkit.protocols.protocol.Protocol._decode_next_layer>`
+    #: DefaultDict[Enum_LinkType, tuple[str, str]]: Protocol index mapping for
+    #: decoding next layer, c.f. :meth:`self._decode_next_layer <pcapkit.protocols.protocol.Protocol._decode_next_layer>`
     #: & :meth:`self._import_next_layer <pcapkit.protocols.protocol.Protocol._import_next_layer>`.
     #: The values should be a tuple representing the module name and class name.
     __proto__ = collections.defaultdict(
         lambda: ('pcapkit.protocols.misc.raw', 'Raw'),
         {
-            RegType_LinkType.ETHERNET: ('pcapkit.protocols.link', 'Ethernet'),
-            RegType_LinkType.IPV4:     ('pcapkit.protocols.internet', 'IPv4'),
-            RegType_LinkType.IPV6:     ('pcapkit.protocols.internet', 'IPv6'),
+            Enum_LinkType.ETHERNET: ('pcapkit.protocols.link', 'Ethernet'),
+            Enum_LinkType.IPV4:     ('pcapkit.protocols.internet', 'IPv4'),
+            Enum_LinkType.IPV6:     ('pcapkit.protocols.internet', 'IPv6'),
         },
-    )
+    )  # type: DefaultDict[Enum_LinkType | int, tuple[str, str]]
 
     ##########################################################################
     # Properties.
@@ -110,7 +108,7 @@ class Frame(Protocol[DataType_Frame]):
         return 16
 
     @property
-    def header(self) -> 'DataType_Header':
+    def header(self) -> 'Data_Header':
         """Global header of the PCAP file."""
         return self._ghdr
 
@@ -119,7 +117,7 @@ class Frame(Protocol[DataType_Frame]):
     ##########################################################################
 
     @classmethod
-    def register(cls, code: 'RegType_LinkType', module: 'str', class_: 'str') -> 'None':
+    def register(cls, code: 'Enum_LinkType', module: 'str', class_: 'str') -> 'None':  # type: ignore[override]
         r"""Register a new protocol class.
 
         Notes:
@@ -132,6 +130,8 @@ class Frame(Protocol[DataType_Frame]):
             class\_: class name
 
         """
+        if code in cls.__proto__:
+            warn(f'protocol {code} already registered, overwriting', RegistryWarning)
         cls.__proto__[code] = (module, class_)
 
     def index(self, name: 'str | Protocol | Type[Protocol]') -> 'int':
@@ -149,42 +149,82 @@ class Frame(Protocol[DataType_Frame]):
         """
         return self._protos.index(name)
 
-    def read(self, length: 'Optional[int]' = None, *,
-             _read: 'bool' = True, **kwargs: 'Any') -> 'DataType_Frame':
-        r"""Read each block after global header.
+    def pack(self, **kwargs: 'Any') -> 'bytes':
+        """Pack (construct) packet data.
+
+        Args:
+            **kwargs: Arbitrary keyword arguments.
+
+        Returns:
+            Constructed packet data.
+
+        Notes:
+            We used a special keyword argument ``__packet__`` to pass the
+            global packet data to underlying methods. This is useful when
+            the packet data is not available in the current instance.
+
+        """
+        self.__header__ = self.make(**kwargs)
+        packet = kwargs.get('__packet__', {})  # packet data
+        packet['byteorder'] = self._ghdr.magic_number.byteorder
+        return self.__header__.pack(packet)
+
+    def unpack(self, length: 'Optional[int]' = None, **kwargs: 'Any') -> 'Data_Frame':
+        """Unpack (parse) packet data.
 
         Args:
             length: Length of packet data.
+            **kwargs: Arbitrary keyword arguments.
+
+        Returns:
+            Parsed packet data.
+
+        Notes:
+            We used a special keyword argument ``__packet__`` to pass the
+            global packet data to underlying methods. This is useful when
+            the packet data is not available in the current instance.
+
+        """
+        if cast('Optional[Schema_Frame]', self.__header__) is None:
+            packet = kwargs.get('__packet__', {})  # packet data
+            packet['bytesorder'] = self._ghdr.magic_number.byteorder
+            self.__header__ = cast('Schema_Frame', self.__schema__.unpack(self._file, length, packet))  # type: ignore[call-arg,misc]
+        return self.read(length, **kwargs)
+
+    def read(self, length: 'Optional[int]' = None, *, _read: 'bool' = True, **kwargs: 'Any') -> 'Data_Frame':
+        r"""Read each block after global header.
+
+        Args:
+            length: Length of data to be read.
             \_read: If the class is called in a parsing scenario.
             **kwargs: Arbitrary keyword arguments.
 
         Returns:
-            DataType_Frame: Parsed packet data.
-
-        Raises:
-            EOFError: If :attr:`self._file <pcapkit.protocols.protocol.Protocol._file>` reaches EOF.
+            Data_Frame: Parsed packet data.
 
         """
+        schema = self.__header__
+
+        _tsss = schema.ts_sec
+        _tsus = schema.ts_usec
+        _ilen = schema.incl_len
+        _olen = schema.orig_len
+
+        with localcontext(prec=64):
+            if self._nsec:
+                _epch = _tsss + decimal.Decimal(_tsus) / 1_000_000_000
+            else:
+                _epch = _tsss + decimal.Decimal(_tsus) / 1_000_000
+        _irat = _epch.as_integer_ratio()
+
         try:
-            _temp = self._read_unpack(4, lilendian=True)
-        except StructError as exc:
-            if exc.eof:
-                raise EOFError  # pylint: disable=raise-missing-from
-            raise
+            _time = datetime.datetime.fromtimestamp(_irat[0] / _irat[1])
+        except ValueError:
+            warn(f'PCAP: invalid timestamp: {_epch}', ProtocolWarning, stacklevel=stacklevel())
+            _time = datetime.datetime.fromtimestamp(0, datetime.timezone.utc)
 
-        _tsss = _temp
-        _tsus = self._read_unpack(4, lilendian=True)
-        _ilen = self._read_unpack(4, lilendian=True)
-        _olen = self._read_unpack(4, lilendian=True)
-
-        if self._nsec:
-            _epch = _tsss + decimal.Decimal(_tsus) / 1_000_000_000
-        else:
-            _epch = _tsss + decimal.Decimal(_tsus) / 1_000_000
-        _time = datetime.datetime.fromtimestamp(float(_epch))
-
-        frame = DataType_Frame(
-            frame_info=DataType_FrameInfo(
+        frame = Data_Frame(
+            frame_info=Data_FrameInfo(
                 ts_sec=_tsss,
                 ts_usec=_tsus,
                 incl_len=_ilen,
@@ -199,28 +239,35 @@ class Frame(Protocol[DataType_Frame]):
 
         if not _read:
             # move backward to the beginning of the packet
-            self._file.seek(-self.length, io.SEEK_CUR)
+            self._file.seek(0, io.SEEK_SET)
         else:
-            # NOTE: We create a copy of the frame packet data here for parsing
-            # scenarios to keep the original packet data intact.
+            # NOTE: We create a copy of the frame data here for parsing
+            # scenarios to keep the original frame data intact.
+            seek_cur = self._file.tell()
 
             # move backward to the beginning of the frame
             self._file.seek(-self.length, io.SEEK_CUR)
 
-            #: bytes: Raw packet data.
-            self._data = self._read_fileng(self.length + frame.len)
-            #: io.BytesIO: Source packet stream.
-            self._file = io.BytesIO(self._data)
+            #: bytes: Raw frame data.
+            self._data = self._read_fileng(self.length + _ilen)
 
-            # move forward to the beginning of frame's first packet
-            self._file.seek(self.length, io.SEEK_CUR)
+            # move backward to the beginning of frame's payload
+            self._file.seek(seek_cur, io.SEEK_SET)
+
+            #: io.BytesIO: Source data stream.
+            self._file = io.BytesIO(self._data)
 
         return self._decode_next_layer(frame, self._ghdr.network, frame.len)
 
-    def make(self, *, timestamp: 'Optional[float | Decimal]' = None,  # type: ignore[override] # pylint: disable=arguments-differ
-             ts_sec: 'Optional[int]' = None, ts_usec: 'Optional[int]' = None,
-             incl_len: 'Optional[int]' = None, orig_len: 'Optional[int]' = None,
-             packet: 'bytes', nanosecond: 'bool' = False, **kwargs: 'Any') -> 'bytes':
+    def make(self,
+             timestamp: 'Optional[float | Decimal | int | dt_type]' = None,
+             ts_sec: 'Optional[int]' = None,
+             ts_usec: 'Optional[int]' = None,
+             incl_len: 'Optional[int]' = None,
+             orig_len: 'Optional[int]' = None,
+             packet: 'bytes | Protocol | Schema' = b'',
+             nanosecond: 'bool' = False,
+             **kwargs: 'Any') -> 'Schema_Frame':
         """Make frame packet data.
 
         Args:
@@ -244,13 +291,12 @@ class Frame(Protocol[DataType_Frame]):
         if orig_len is None:
             orig_len = len(packet)
 
-        # make packet
-        return b'%s%s%s%s%s' % (
-            self._make_pack(ts_sec, size=4, lilendian=True),
-            self._make_pack(ts_usec, size=4, lilendian=True),
-            self._make_pack(incl_len, size=4, lilendian=True),
-            self._make_pack(orig_len, size=4, lilendian=True),
-            packet[:incl_len],
+        return Schema_Frame(
+            ts_sec=ts_sec,
+            ts_usec=ts_usec,
+            incl_len=incl_len,
+            orig_len=orig_len,
+            packet=packet,
         )
 
     ##########################################################################
@@ -258,14 +304,14 @@ class Frame(Protocol[DataType_Frame]):
     ##########################################################################
 
     @overload  # type: ignore[override]
-    def __post_init__(self, file: 'BinaryIO', length: 'Optional[int]' = ..., *,  # pylint: disable=arguments-differ
-                      num: 'int', header: 'DataType_Header', **kwargs: 'Any') -> 'None': ...
+    def __post_init__(self, file: 'IO[bytes] | bytes', length: 'Optional[int]' = ..., *,  # pylint: disable=arguments-differ
+                      num: 'int', header: 'Data_Header', **kwargs: 'Any') -> 'None': ...
     @overload
-    def __post_init__(self, *, num: 'int', header: 'DataType_Header',  # pylint: disable=arguments-differ
+    def __post_init__(self, *, num: 'int', header: 'Data_Header',  # pylint: disable=arguments-differ
                       **kwargs: 'Any') -> 'None': ...
 
-    def __post_init__(self, file: 'Optional[BinaryIO]' = None, length: 'Optional[int]' = None, *,  # pylint: disable=arguments-differ
-                      num: 'int', header: 'DataType_Header', **kwargs: 'Any') -> 'None':
+    def __post_init__(self, file: 'Optional[IO[bytes] | bytes]' = None, length: 'Optional[int]' = None, *,  # pylint: disable=arguments-differ
+                      num: 'int', header: 'Data_Header', **kwargs: 'Any') -> 'None':
         """Initialisation.
 
         Args:
@@ -292,16 +338,16 @@ class Frame(Protocol[DataType_Frame]):
         if file is None:
             _read = False
             #: bytes: Raw packet data.
-            self._data = self.make(**kwargs)
+            self._data = self.pack(**kwargs)
             #: io.BytesIO: Source packet stream.
             self._file = io.BytesIO(self._data)
         else:
             _read = True
             #: io.BytesIO: Source packet stream.
-            self._file = file
+            self._file = io.BytesIO(file) if isinstance(file, bytes) else file
 
         #: pcapkit.corekit.infoclass.Info: Parsed packet data.
-        self._info = self.read(length, _read=_read, **kwargs)
+        self._info = self.unpack(length, _read=_read, **kwargs)
 
     def __length_hint__(self) -> 'Literal[16]':
         """Return an estimated length for the object."""
@@ -331,7 +377,26 @@ class Frame(Protocol[DataType_Frame]):
     # Utilities.
     ##########################################################################
 
-    def _make_timestamp(self, timestamp: 'Optional[float | Decimal]' = None, ts_sec: 'Optional[int]' = None,
+    @classmethod
+    def _make_data(cls, data: 'Data_Frame') -> 'dict[str, Any]':  # type: ignore[override]
+        """Create key-value pairs from ``data`` for protocol construction.
+
+        Args:
+            data: protocol data
+
+        Returns:
+            Key-value pairs for protocol construction.
+
+        """
+        return {
+            'ts_src': data.frame_info.ts_sec,
+            'ts_usec': data.frame_info.ts_usec,
+            'incl_len': data.frame_info.incl_len,
+            'orig_len': data.frame_info.orig_len,
+            'packet': cls._make_payload(data),
+        }
+
+    def _make_timestamp(self, timestamp: 'Optional[float | Decimal | dt_type | int]' = None, ts_sec: 'Optional[int]' = None,
                         ts_usec: 'Optional[int]' = None, nanosecond: 'bool' = False) -> 'tuple[int, int]':
         """Make timestamp.
 
@@ -345,36 +410,49 @@ class Frame(Protocol[DataType_Frame]):
             Second and microsecond/nanosecond value of timestamp.
 
         """
-        if timestamp is None:
-            if py37 and nanosecond:
-                timestamp = decimal.Decimal(time.time_ns()) / 1_000_000_000
+        with localcontext(prec=64):
+            if timestamp is None:
+                if py37 and nanosecond:
+                    timestamp = decimal.Decimal(time.time_ns()) / 1_000_000_000
+                else:
+                    timestamp = decimal.Decimal(time.time())
             else:
-                timestamp = decimal.Decimal(time.time())
-        else:
-            timestamp = decimal.Decimal(timestamp)
+                if isinstance(timestamp, datetime.datetime):
+                    timestamp = timestamp.timestamp()
+                timestamp = decimal.Decimal(timestamp)
 
         if ts_sec is None:
             ts_sec = int(timestamp)
 
         if ts_usec is None:
-            ts_usec = int(timestamp - ts_sec) * (1_000_000_000 if nanosecond else 1_000_000)
+            ts_usec = int((timestamp - ts_sec) * (1_000_000_000 if nanosecond else 1_000_000))
 
         return ts_sec, ts_usec
 
-    def _decode_next_layer(self, dict_: 'DataType_Frame', proto: 'Optional[int]' = None,
-                           length: 'Optional[int]' = None) -> 'DataType_Frame':  # pylint: disable=arguments-differ
+    def _decode_next_layer(self, dict_: 'Data_Frame', proto: 'Optional[int]' = None,
+                           length: 'Optional[int]' = None, *, packet: 'Optional[dict[str, Any]]' = None) -> 'Data_Frame':  # pylint: disable=arguments-differ
         r"""Decode next layer protocol.
 
         Arguments:
             dict\_: info buffer
             proto: next layer protocol index
             length: valid (*non-padding*) length
+            packet: packet info (passed from :meth:`self.unpack <pcapkit.protocols.protocol.Protocol.unpack>`)
 
         Returns:
-            dict: current protocol with packet extracted
+            Current protocol with packet extracted.
+
+        Notes:
+            We added a new key ``__next_type__`` to ``dict_`` to store the
+            next layer protocol type, and a new key ``__next_name__`` to
+            store the next layer protocol name. These two keys will **NOT**
+            be included when :meth:`Info.to_dict <pcapkit.corekit.infoclass.Info.to_dict>` is called.
+
+            We also added a new key ``protocols`` to ``dict_`` to store the
+            protocol chain of the current packet (frame).
 
         """
-        next_ = self._import_next_layer(proto, length)  # type: ignore[misc,call-arg]
+        next_ = cast('Protocol', self._import_next_layer(proto, length, packet=packet))  # type: ignore[misc,call-arg,redundant-cast]
         info, chain = next_.info, next_.protochain
 
         # make next layer protocol name
@@ -382,10 +460,12 @@ class Frame(Protocol[DataType_Frame]):
         # proto = next_.__class__.__name__
 
         # write info and protocol chain into dict
-        dict_.__update__([
-            (layer, info),
-            ('protocols', chain.chain),
-        ])
+        dict_.__update__({
+            layer: info,
+            'protocols': chain.chain if chain else '',
+            '__next_type__': type(next_),
+            '__next_name__': layer,
+        })
         self._next = next_  # pylint: disable=attribute-defined-outside-init
         self._protos = chain  # pylint: disable=attribute-defined-outside-init
         return dict_
