@@ -2,6 +2,8 @@
 """Root Protocol
 ===================
 
+.. module:: pcapkit.protocols.protocol
+
 :mod:`pcapkit.protocols.protocol` contains
 :class:`~pcapkit.protocols.protocol.Protocol` only, which is
 an abstract base class for all protocol family, with pre-defined
@@ -28,43 +30,57 @@ import chardet
 
 from pcapkit.corekit.infoclass import Info
 from pcapkit.corekit.protochain import ProtoChain
-from pcapkit.protocols.data.protocol import Packet as DataType_Packet
+from pcapkit.protocols import data as data_module
+from pcapkit.protocols import schema as schema_module
+from pcapkit.protocols.data.data import Data
+from pcapkit.protocols.data.misc.raw import Raw as Data_Raw
+from pcapkit.protocols.data.protocol import Packet as Data_Packet
+from pcapkit.protocols.schema.misc.raw import Raw as Schema_Raw
+from pcapkit.protocols.schema.schema import Schema
 from pcapkit.utilities.compat import cached_property
 from pcapkit.utilities.decorators import beholder, seekset
-from pcapkit.utilities.exceptions import (ProtocolNotFound, ProtocolNotImplemented, StructError,
-                                          UnsupportedCall)
+from pcapkit.utilities.exceptions import (ProtocolNotFound, ProtocolNotImplemented, RegistryError,
+                                          StructError, UnsupportedCall)
 
 if TYPE_CHECKING:
     from enum import IntEnum as StdlibEnum
-    from typing import Any, BinaryIO, DefaultDict, Optional, Type
+    from typing import IO, Any, DefaultDict, Optional, Type
 
     from aenum import IntEnum as AenumEnum
-    from typing_extensions import Literal
+    from typing_extensions import Literal, Self
 
 __all__ = ['Protocol']
 
-PT = TypeVar('PT', bound='Info')
+PT = TypeVar('PT', bound='Data')
+ST = TypeVar('ST', bound='Schema')
 
 # readable characters' order list
 readable = [ord(char) for char in filter(lambda char: not char.isspace(), string.printable)]
 
 
-class Protocol(Generic[PT], metaclass=abc.ABCMeta):
+class Protocol(Generic[PT, ST], metaclass=abc.ABCMeta):
     """Abstract base class for all protocol family."""
 
-    #: Parsed packet data.
-    _info: 'PT'
-    #: Raw packet data.
-    _data: 'bytes'
-    #: Source packet stream.
-    _file: 'BinaryIO'
-    #： Next layer protocol instance.
-    _next: 'Protocol'
-    #: Protocol chain instance.
-    _protos: 'ProtoChain'
+    if TYPE_CHECKING:
+        #: Parsed packet data.
+        _info: 'PT'
+        #: Raw packet data.
+        _data: 'bytes'
+        #: Source packet stream.
+        _file: 'IO[bytes]'
+        #: Next layer protocol instance.
+        _next: 'Protocol'
+        #: Protocol chain instance.
+        _protos: 'ProtoChain'
 
-    # Internal data storage for cached properties.
-    __cached__: 'dict[str, Any]'
+        # Internal data storage for cached properties.
+        __cached__: 'dict[str, Any]'
+        #: Protocol packet data definition.
+        __data__: 'Type[PT]'
+        #: Protocol header schema definition.
+        __schema__: 'Type[ST]'
+        #: Protocol header schema instance.
+        __header__: 'ST'
 
     ##########################################################################
     # Defaults.
@@ -147,16 +163,22 @@ class Protocol(Generic[PT], metaclass=abc.ABCMeta):
 
     # packet data
     @cached_property
-    def packet(self) -> 'DataType_Packet':
-        """DataType_Packet data of the protocol."""
+    def packet(self) -> 'Data_Packet':
+        """Data_Packet data of the protocol."""
         try:
             return self._read_packet(header=self.length)
         except UnsupportedCall:
-            return DataType_Packet(
+            return Data_Packet(
                 header=b'',
                 payload=self._read_packet(),
             )
 
+    # schema data
+    @cached_property
+    def schema(self) -> 'ST':
+        """Schema data of the protocol."""
+        return self.__header__
+
     ##########################################################################
     # Methods.
     ##########################################################################
@@ -188,17 +210,57 @@ class Protocol(Generic[PT], metaclass=abc.ABCMeta):
         """
 
     @abc.abstractmethod
-    def make(self, **kwargs: 'Any') -> 'bytes':
+    def make(self, **kwargs: 'Any') -> 'ST':
         """Make (construct) packet data.
 
         Args:
             **kwargs: Arbitrary keyword arguments.
 
         Returns:
-            bytes: Constructed packet data.
+            Curated protocol schema data.
 
         """
 
+    def pack(self, **kwargs: 'Any') -> 'bytes':
+        """Pack (construct) packet data.
+
+        Args:
+            **kwargs: Arbitrary keyword arguments.
+
+        Returns:
+            Constructed packet data.
+
+        Notes:
+            We used a special keyword argument ``__packet__`` to pass the
+            global packet data to underlying methods. This is useful when
+            the packet data is not available in the current instance.
+
+        """
+        self.__header__ = self.make(**kwargs)
+        packet = kwargs.get('__packet__', {})  # packet data
+        return self.__header__.pack(packet)
+
+    def unpack(self, length: 'Optional[int]' = None, **kwargs: 'Any') -> 'PT':
+        """Unpack (parse) packet data.
+
+        Args:
+            length: Length of packet data.
+            **kwargs: Arbitrary keyword arguments.
+
+        Returns:
+            Parsed packet data.
+
+        Notes:
+            We used a special keyword argument ``__packet__`` to pass the
+            global packet data to underlying methods. This is useful when
+            the packet data is not available in the current instance.
+
+        """
+        if cast('Optional[ST]', self.__header__) is None:
+            packet = kwargs.get('__packet__', {})  # packet data
+            self.__header__ = cast('ST', self.__schema__.unpack(self._file, length, packet))  # type: ignore[call-arg,misc]
+        return self.read(length, **kwargs)
+
     @staticmethod
     def decode(byte: bytes, *, encoding: 'Optional[str]' = None,
                errors: 'Literal["strict", "ignore", "replace"]' = 'strict') -> 'str':
@@ -232,7 +294,7 @@ class Protocol(Generic[PT], metaclass=abc.ABCMeta):
             return byte.decode('unicode_escape', errors='replace')
 
     @staticmethod
-    def unquote(url: str, *, encoding : 'str' = 'utf-8',
+    def unquote(url: str, *, encoding: 'str' = 'utf-8',
                 errors: 'Literal["strict", "ignore", "replace"]' = 'replace') -> 'str':
         """Unquote URLs into readable format.
 
@@ -286,15 +348,7 @@ class Protocol(Generic[PT], metaclass=abc.ABCMeta):
         else:
             from pcapkit.protocols import __proto__ as protocols_registry  # pylint: disable=import-outside-toplevel # isort: skip
 
-            if False:
-                proto = NotImplemented
-
-            def _walrus_wrapper_proto_f8c5d7a7bb0f49aa835ac040bfeb3ae2(expr):
-                """Wrapper function for assignment expression."""
-                nonlocal proto
-                proto = expr
-                return proto
-            if (_walrus_wrapper_proto_f8c5d7a7bb0f49aa835ac040bfeb3ae2(protocols_registry.get(value.upper()))) is not None:
+            if (proto := protocols_registry.get(value.upper())) is not None:
                 comp = (proto, *(name.upper() for name in proto.id()))
             else:
                 comp = (value.upper(),)
@@ -322,9 +376,9 @@ class Protocol(Generic[PT], metaclass=abc.ABCMeta):
             report = protocol(payload_io, len(payload), **kwargs)  # type: ignore[abstract]
         except Exception as exc:
             if isinstance(exc, StructError) and exc.eof:  # pylint: disable=no-member
-                from pcapkit.protocols.misc.null import NoPayload as protocol  # type: ignore[no-redef] # pylint: disable=import-outside-toplevel # isort: skip
+                from pcapkit.protocols.misc.null import NoPayload as protocol  # pylint: disable=import-outside-toplevel # isort: skip
             else:
-                from pcapkit.protocols.misc.raw import Raw as protocol  # type: ignore[no-redef] # pylint: disable=import-outside-toplevel # isort: skip
+                from pcapkit.protocols.misc.raw import Raw as protocol  # pylint: disable=import-outside-toplevel # isort: skip
             # error = traceback.format_exc(limit=1).strip().rsplit(os.linesep, maxsplit=1)[-1]
 
             # log error
@@ -333,25 +387,89 @@ class Protocol(Generic[PT], metaclass=abc.ABCMeta):
             report = protocol(payload_io, len(payload), **kwargs)  # type: ignore[abstract]
         return report
 
+    @classmethod
+    def register(cls, code: 'int', module: 'str', class_: 'str') -> 'None':
+        r"""Register a new protocol class.
+
+        Notes:
+            The full qualified class name of the new protocol class
+            should be as ``{module}.{class_}``.
+
+        Arguments:
+            code: protocol code
+            module: module name
+            class\_: class name
+
+        """
+        protocol = getattr(importlib.import_module(module), class_)
+        if not issubclass(protocol, Protocol):
+            raise RegistryError('protocol must be a Protocol subclass')
+        cls.__proto__[code] = (module, class_)
+
+    @classmethod
+    def from_schema(cls, schema: 'ST | dict[str, Any]') -> 'Self':
+        """Create protocol instance from schema.
+
+        Args:
+            schema: Protocol schema.
+
+        Returns:
+            Protocol instance.
+
+        """
+        if not isinstance(schema, Schema):
+            schema = cast('ST', cls.__schema__.from_dict(schema))
+
+        self = cls.__new__(cls)
+        self.__header__ = schema
+
+        # initialize protocol instance
+        self.__init__(bytes(schema), len(schema))  # type: ignore[misc]
+
+        return self
+
+    @classmethod
+    def from_data(cls, data: 'PT | dict[str, Any]') -> 'Self':
+        """Create protocol instance from data.
+
+        Args:
+            data: Protocol data.
+
+        Returns:
+            Protocol instance.
+
+        """
+        if not isinstance(data, Data):
+            data = cast('PT', cls.__data__.from_dict(data))
+
+        self = cls.__new__(cls)
+        kwargs = self._make_data(data)
+
+        # initialize protocol instance
+        self.__init__(**kwargs)  # type: ignore[misc]
+
+        return self
+
     ##########################################################################
     # Data models.
     ##########################################################################
 
-    def __new__(cls, *args: 'Any', **kwargs: 'Any') -> 'Protocol[PT]':  # pylint: disable=unused-argument
+    def __new__(cls, *args: 'Any', **kwargs: 'Any') -> 'Self':  # pylint: disable=unused-argument
         self = super().__new__(cls)
 
         # NOTE: Assign this attribute after ``__new__`` to avoid shared memory
         # reference between instances.
         self.__cached__ = {}
+        self.__header__ = None  # type: ignore[assignment]
 
         return self
 
     @overload
-    def __init__(self, file: 'BinaryIO', length: 'Optional[int]' = ..., **kwargs: 'Any') -> 'None': ...
+    def __init__(self, file: 'IO[bytes] | bytes', length: 'Optional[int]' = ..., **kwargs: 'Any') -> 'None': ...
     @overload
     def __init__(self, **kwargs: 'Any') -> 'None': ...
 
-    def __init__(self, file: 'Optional[BinaryIO]' = None, length: 'Optional[int]' = None, **kwargs: 'Any') -> 'None':
+    def __init__(self, file: 'Optional[IO[bytes] | bytes]' = None, length: 'Optional[int]' = None, **kwargs: 'Any') -> 'None':
         """Initialisation.
 
         Args:
@@ -379,11 +497,11 @@ class Protocol(Generic[PT], metaclass=abc.ABCMeta):
         self.__post_init__(file, length, **kwargs)  # type: ignore[arg-type]
 
     @overload
-    def __post_init__(self, file: 'BinaryIO', length: 'Optional[int]' = ..., **kwargs: 'Any') -> 'None': ...
+    def __post_init__(self, file: 'IO[bytes] | bytes', length: 'Optional[int]' = ..., **kwargs: 'Any') -> 'None': ...
     @overload
     def __post_init__(self, **kwargs: 'Any') -> 'None': ...
 
-    def __post_init__(self, file: 'Optional[BinaryIO]' = None,
+    def __post_init__(self, file: 'Optional[IO[bytes] | bytes]' = None,
                       length: 'Optional[int]' = None, **kwargs: 'Any') -> 'None':
         """Post initialisation hook.
 
@@ -398,16 +516,48 @@ class Protocol(Generic[PT], metaclass=abc.ABCMeta):
 
         """
         if file is None:
-            _data = self.make(**kwargs)
+            _data = self.pack(**kwargs)
         else:
-            _data = file.read(length)  # type: ignore[arg-type]
+            _data = file if isinstance(file, bytes) else file.read(length)  # type: ignore[arg-type]
 
         #: bytes: Raw packet data.
         self._data = _data
         #: io.BytesIO: Source packet stream.
         self._file = io.BytesIO(self._data)
-        #: pcapkit.corekit.infoclass.Info: Parsed packet data.
-        self._info = self.read(length, **kwargs)
+        #: pcapkit.protocols.data.data.Data: Parsed packet data.
+        self._info = self.unpack(length, **kwargs)
+
+    def __init_subclass__(cls, /, schema: 'Optional[Type[ST]]' = None,
+                          data: 'Optional[Type[PT]]' = None, **kwargs: 'Any') -> 'None':
+        """Initialisation for subclasses.
+
+        Args:
+            schema: Schema class.
+            data: Data class.
+            **kwargs: Arbitrary keyword arguments.
+
+        This method is called when a subclass of :class:`Protocol` is defined.
+        It is used to set the :attr:`self.__schema__ <pcapkit.protocols.protocol.Protocol.__schema__>`
+        attribute of the subclass.
+
+        Notes:
+            When ``schema`` and/or ``data`` is not specified, the method will first
+            try to find the corresponding class in the
+            :mod:`~pcapkit.protocols.schema` and :mod:`~pcapkit.protocols.data`
+            modules respectively. If the class is not found, the default
+            :class:`~pcapkit.protocols.schema.schema.Schema_Raw` and
+            :class:`~pcapkit.protocols.data.data.Data_Raw` classes will be used.
+
+        """
+        super().__init_subclass__(**kwargs)
+
+        if schema is None:
+            schema = cast('Type[ST]', getattr(schema_module, cls.__name__, Schema_Raw))
+        if data is None:
+            data = cast('Type[PT]', getattr(data_module, cls.__name__, Data_Raw))
+
+        cls.__schema__ = schema
+        cls.__data__ = data
 
     def __repr__(self) -> 'str':
         """Returns representation of parsed protocol data.
@@ -417,17 +567,7 @@ class Protocol(Generic[PT], metaclass=abc.ABCMeta):
             <Frame alias='...' frame=(..., packet=b'...', sethernet=..., protocols='Ethernet:IPv6:Raw')>
 
         """
-
-        if False:
-            cached = NotImplemented
-
-        def _walrus_wrapper_cached_4596c47ee71d44ed897423a75404d853(expr):
-            """Wrapper function for assignment expression."""
-            nonlocal cached
-            cached = expr
-            return cached
-
-        if (_walrus_wrapper_cached_4596c47ee71d44ed897423a75404d853(self.__cached__.get('__repr__'))) is not None:
+        if (cached := self.__cached__.get('__repr__')) is not None:
             return cached
 
         name = type(self).__name__
@@ -458,17 +598,7 @@ class Protocol(Generic[PT], metaclass=abc.ABCMeta):
             5e 60 d9 6b 97                                               ^`.k.
 
         """
-
-        if False:
-            cached = NotImplemented
-
-        def _walrus_wrapper_cached_2d82571979a34f8e881b5cfb9ae2b256(expr):
-            """Wrapper function for assignment expression."""
-            nonlocal cached
-            cached = expr
-            return cached
-
-        if (_walrus_wrapper_cached_2d82571979a34f8e881b5cfb9ae2b256(self.__cached__.get('__str__'))) is not None:
+        if (cached := self.__cached__.get('__str__')) is not None:
             return cached
 
         hexbuf = ' '.join(textwrap.wrap(self._data.hex(), 2))
@@ -492,17 +622,7 @@ class Protocol(Generic[PT], metaclass=abc.ABCMeta):
 
     def __len__(self) -> 'int':
         """Total length of corresponding protocol."""
-
-        if False:
-            cached = NotImplemented
-
-        def _walrus_wrapper_cached_fbca46d3aaba4cb78d3a816259c99e74(expr):
-            """Wrapper function for assignment expression."""
-            nonlocal cached
-            cached = expr
-            return cached
-
-        if (_walrus_wrapper_cached_fbca46d3aaba4cb78d3a816259c99e74(self.__cached__.get('__len__'))) is not None:
+        if (cached := self.__cached__.get('__len__')) is not None:
             return cached
 
         # cache and return
@@ -514,7 +634,7 @@ class Protocol(Generic[PT], metaclass=abc.ABCMeta):
     def __length_hint__(self) -> 'Optional[int]':
         """Return an estimated length for the object."""
 
-    def __iter__(self) -> 'BinaryIO':
+    def __iter__(self) -> 'IO[bytes]':
         """Iterate through :attr:`self._data <pcapkit.protocols.protocol.Protocol._data>`."""
         return io.BytesIO(self._data)
 
@@ -624,6 +744,18 @@ class Protocol(Generic[PT], metaclass=abc.ABCMeta):
     # Utilities.
     ##########################################################################
 
+    def _get_payload(self) -> 'bytes':
+        """Get payload of :attr:`self.__header__ <Protocol.__header__>`.
+
+        Returns:
+            Payload of :attr:`self.__header__ <Protocol.__header__>` as :obj:`bytes`.
+
+        See Also:
+            This is a wrapper function for :meth:`pcapkit.protocols.schema.Schema.get_payload`.
+
+        """
+        return self.__header__.get_payload()
+
     def _read_protos(self, size: int) -> 'Optional[StdlibEnum | AenumEnum]':  # pylint: disable=unused-argument
         """Read next layer protocol type.
 
@@ -719,11 +851,11 @@ class Protocol(Generic[PT], metaclass=abc.ABCMeta):
     @overload
     def _read_packet(self, *, header: 'int', payload: 'Optional[int]' = ..., discard: 'Literal[True]') -> 'bytes': ...
     @overload
-    def _read_packet(self, *, header: 'int', payload: 'Optional[int]' = ..., discard: 'Literal[False]' = ...) -> 'DataType_Packet': ...  # pylint: disable=line-too-long
+    def _read_packet(self, *, header: 'int', payload: 'Optional[int]' = ..., discard: 'Literal[False]' = ...) -> 'Data_Packet': ...  # pylint: disable=line-too-long
 
     @seekset  # type: ignore[misc]
     def _read_packet(self, length: 'Optional[int]' = None, *, header: 'Optional[int]' = None,
-                     payload: 'Optional[int]' = None, discard: bool = False) -> 'bytes | DataType_Packet':
+                     payload: 'Optional[int]' = None, discard: bool = False) -> 'bytes | Data_Packet':
         """Read raw packet data.
 
         Arguments:
@@ -736,7 +868,7 @@ class Protocol(Generic[PT], metaclass=abc.ABCMeta):
         * If ``discard`` is set as :data:`True`, returns the packet body (in
           :obj:`bytes`) only.
         * Otherwise, returns the header and payload data as
-          :class:`~pcapkit.protocols.data.protocol.DataType_Packet` object.
+          :class:`~pcapkit.protocols.data.protocol.Data_Packet` object.
 
         """
         if header is not None:
@@ -744,7 +876,7 @@ class Protocol(Generic[PT], metaclass=abc.ABCMeta):
             data_payload = self._read_fileng(payload)
             if discard:
                 return data_payload
-            return DataType_Packet(
+            return Data_Packet(
                 header=data_header,
                 payload=data_payload
             )
@@ -829,6 +961,11 @@ class Protocol(Generic[PT], metaclass=abc.ABCMeta):
                     reversed: 'Literal[True]',  # pylint: disable=redefined-builtin
                     pack: 'Literal[True]', size: 'int' = ..., signed: 'bool' = ...,
                     lilendian: 'bool' = ...) -> 'bytes': ...
+    @overload
+    @classmethod
+    def _make_index(cls, name: 'str | int | StdlibEnum | AenumEnum', default: 'Optional[int]' = ..., *,
+                    namespace: 'Optional[dict[str, int] | dict[int, str] | Type[StdlibEnum] | Type[AenumEnum]]' = ...,
+                    reversed: 'bool' = ..., pack: 'Literal[False]' = ...) -> 'int': ...
 
     @classmethod
     def _make_index(cls, name: 'str | int | StdlibEnum | AenumEnum', default: 'Optional[int]' = None, *,
@@ -857,7 +994,7 @@ class Protocol(Generic[PT], metaclass=abc.ABCMeta):
                 and ``default`` is :data:`None`.
 
         """
-        if isinstance(name, (enum.IntEnum, aenum.IntEnum)):
+        if isinstance(name, (enum.Enum, aenum.Enum)):
             index = cast('int', name.value)
         elif isinstance(name, int):
             index = name
@@ -885,19 +1022,71 @@ class Protocol(Generic[PT], metaclass=abc.ABCMeta):
             return cls._make_pack(index, size=size, signed=signed, lilendian=lilendian)
         return index
 
-    def _decode_next_layer(self, dict_: 'PT', proto: 'int', length: 'Optional[int]' = None) -> 'PT':
+    @classmethod
+    def _make_data(cls, data: 'Data') -> 'dict[str, Any]':
+        """Create key-value pairs from ``data`` for protocol construction.
+
+        Args:
+            data: protocol data
+
+        Returns:
+            Key-value pairs for protocol construction.
+
+        """
+        return data.to_dict()
+
+    @classmethod
+    def _make_payload(cls, data: 'Data') -> 'Protocol':
+        """Create payload from ``data`` for protocol construction.
+
+        This method uses ``__next_type__`` and ``__next_name__`` to
+        determine the payload type and name. If either of them is
+        :data:`None`, a :class:`~pcapkit.protocols.misc.null.NoPayload`
+        instance will be returned. Otherwise, the payload will be
+        constructed by :meth:`Protocol.from_data <pcapkit.protocols.protocol.Protocol.from_data>`.
+
+        Args:
+            data: protocol data
+
+        Returns:
+            Payload for protocol construction.
+
+        """
+        proto = cast('Optional[Type[Protocol]]', data.get('__next_type__'))
+        if proto is None or not (isinstance(proto, type) and issubclass(proto, Protocol)):
+            from pcapkit.protocols.misc.null import \
+                NoPayload  # pylint: disable=import-outside-toplevel
+            return NoPayload()
+
+        name = cast('Optional[str]', data.get('__next_name__'))
+        if name is None:
+            from pcapkit.protocols.misc.null import \
+                NoPayload  # pylint: disable=import-outside-toplevel
+            return NoPayload()
+
+        return proto.from_data(data[name])
+
+    def _decode_next_layer(self, dict_: 'PT', proto: 'int', length: 'Optional[int]' = None, *,
+                           packet: 'Optional[dict[str, Any]]' = None) -> 'PT':
         r"""Decode next layer protocol.
 
         Arguments:
             dict\_: info buffer
             proto: next layer protocol index
             length: valid (*non-padding*) length
+            packet: packet info (passed from :meth:`unpack`)
 
         Returns:
             Current protocol with next layer extracted.
 
+        Notes:
+            We added a new key ``__next_type__`` to ``dict_`` to store the
+            next layer protocol type, and a new key ``__next_name__`` to
+            store the next layer protocol name. These two keys will **NOT**
+            be included when :meth:`Info.to_dict <pcapkit.corekit.infoclass.Info.to_dict>` is called.
+
         """
-        next_ = self._import_next_layer(proto, length)  # type: ignore[call-arg,misc]
+        next_ = cast('Protocol', self._import_next_layer(proto, length, packet=packet))  # type: ignore[misc,call-arg,redundant-cast]
         info, chain = next_.info, next_.protochain
 
         # make next layer protocol name
@@ -905,18 +1094,24 @@ class Protocol(Generic[PT], metaclass=abc.ABCMeta):
         # proto = next_.__class__.__name__
 
         # write info and protocol chain into dict
-        dict_.__update__([(layer, info)])
+        dict_.__update__({
+            layer: info,
+            '__next_type__': type(next_),
+            '__next_name__': layer,
+        })
         self._next = next_  # pylint: disable=attribute-defined-outside-init
         self._protos = ProtoChain(self.__class__, self.alias, basis=chain)  # pylint: disable=attribute-defined-outside-init
         return dict_
 
     @beholder
-    def _import_next_layer(self, proto: 'int', length: 'Optional[int]' = None) -> 'Protocol':
+    def _import_next_layer(self, proto: 'int', length: 'Optional[int]' = None, *,
+                           packet: 'Optional[dict[str, Any]]' = None) -> 'Protocol':
         """Import next layer extractor.
 
         Arguments:
             proto: next layer protocol index
             length: valid (*non-padding*) length
+            packet: packet info (passed from :meth:`unpack`)
 
         Returns:
             Instance of next layer.
@@ -925,31 +1120,25 @@ class Protocol(Generic[PT], metaclass=abc.ABCMeta):
         if TYPE_CHECKING:
             protocol: 'Type[Protocol]'
 
-        if length is not None and length == 0:
-            from pcapkit.protocols.misc.null import NoPayload as protocol  # type: ignore[no-redef] # isort: skip # pylint: disable=import-outside-toplevel
+        file_ = self._get_payload()
+        if length is None:
+            length = len(file_)
+
+        if length == 0:
+            from pcapkit.protocols.misc.null import NoPayload as protocol  # isort: skip # pylint: disable=import-outside-toplevel
         elif self._sigterm:
-            from pcapkit.protocols.misc.raw import Raw as protocol  # type: ignore[no-redef] # isort: skip # pylint: disable=import-outside-toplevel
+            from pcapkit.protocols.misc.raw import Raw as protocol  # isort: skip # pylint: disable=import-outside-toplevel
         else:
             module, name = self.__proto__[proto]
             protocol = cast('Type[Protocol]', getattr(importlib.import_module(module), name))
 
-        next_ = protocol(self._file, length, alias=proto,
+        next_ = protocol(file_, length, alias=proto, packet=packet,
                          layer=self._exlayer, protocol=self._exproto)  # type: ignore[abstract]
         return next_
 
     def _check_term_threshold(self) -> bool:
         """Check if reached termination threshold."""
-
-        if False:
-            layer = NotImplemented
-
-        def _walrus_wrapper_layer_0653bad64515433787d4d8eb4e5bddd0(expr):
-            """Wrapper function for assignment expression."""
-            nonlocal layer
-            layer = expr
-            return layer
-
-        if self._exlayer is None or (_walrus_wrapper_layer_0653bad64515433787d4d8eb4e5bddd0(self.__layer__)) is None:
+        if self._exlayer is None or (layer := self.__layer__) is None:
             layer_match = False
         else:
             layer_match = layer.upper() == self._exlayer.upper()
