@@ -1,6 +1,8 @@
 # -*- coding: utf-8 -*-
-""":mod:`DPKT <dpkt>` Tools
-==============================
+"""DPKT Tools
+================
+
+.. module:: pcapkit.toolkit.dpkt
 
 :mod:`pcapkit.toolkit.dpkt` contains all you need for
 :mod:`pcapkit` handy usage with `DPKT`_ engine. All reforming
@@ -13,10 +15,10 @@ its caller.
 import ipaddress
 from typing import TYPE_CHECKING, cast
 
-from pcapkit.const.reg.transtype import TransType as RegType_TransType
-from pcapkit.foundation.reassembly.ip import Packet as IP_Packet
-from pcapkit.foundation.reassembly.tcp import Packet as TCP_Packet
-from pcapkit.foundation.traceflow import Packet as TF_Packet
+from pcapkit.const.reg.transtype import TransType as Enum_TransType
+from pcapkit.foundation.reassembly.data.ip import Packet as IP_Packet
+from pcapkit.foundation.reassembly.data.tcp import Packet as TCP_Packet
+from pcapkit.foundation.traceflow.data.tcp import Packet as TF_TCP_Packet
 
 if TYPE_CHECKING:
     from ipaddress import IPv4Address, IPv6Address
@@ -27,7 +29,7 @@ if TYPE_CHECKING:
     from dpkt.ip6 import IP6, IP6FragmentHeader
     from dpkt.tcp import TCP
 
-    from pcapkit.const.reg.linktype import LinkType as RegType_LinkType
+    from pcapkit.const.reg.linktype import LinkType as Enum_LinkType
 
 __all__ = [
     'ipv6_hdr_len', 'packet2chain', 'packet2dict',
@@ -79,7 +81,7 @@ def packet2chain(packet: 'Packet') -> 'str':
 
 
 def packet2dict(packet: 'Packet', timestamp: 'float', *,
-                data_link: 'RegType_LinkType') -> 'dict[str, Any]':
+                data_link: 'Enum_LinkType') -> 'dict[str, Any]':
     """Convert DPKT packet into :obj:`dict`.
 
     Args:
@@ -121,10 +123,10 @@ def ipv4_reassembly(packet: 'Packet', *, count: 'int' = -1) -> 'IP_Packet[IPv4Ad
           if it contains IPv4 layer (:class:`dpkt.ip.IP`) and the **DF** (:attr:`dpkt.ip.IP.df`)
           flag is ``False``.
         * If the ``packet`` can be reassembled, then the :obj:`dict` mapping of data for IPv4
-          reassembly (:term:`ipv4.packet`) will be returned; otherwise, returns :data:`None`.
+          reassembly (:term:`reasm.ipv4.packet`) will be returned; otherwise, returns :data:`None`.
 
     See Also:
-        :class:`pcapkit.foundation.reassembly.ipv4.IPv4Reassembly`
+        :class:`pcapkit.foundation.reassembly.ipv4.IPv4`
 
     """
     ipv4 = getattr(packet, 'ip', None)  # type: Optional[IP]
@@ -139,7 +141,7 @@ def ipv4_reassembly(packet: 'Packet', *, count: 'int' = -1) -> 'IP_Packet[IPv4Ad
                 cast('IPv4Address',
                      ipaddress.ip_address(ipv4.dst)),           # destination IP address
                 ipv4.id,                                        # identification
-                RegType_TransType.get(ipv4.p).name,             # payload protocol type
+                Enum_TransType.get(ipv4.p).name,             # payload protocol type
             ),
             num=count,                                          # original packet range number
             fo=ipv4.off,                                        # fragment offset
@@ -167,10 +169,10 @@ def ipv6_reassembly(packet: 'Packet', *, count: 'int' = -1) -> 'IP_Packet[IPv6Ad
           if it contains IPv6 layer (:class:`dpkt.ip6.IP6`) and IPv6 Fragment header
           (:rfc:`2460#section-4.5`, :class:`dpkt.ip6.IP6FragmentHeader`).
         * If the ``packet`` can be reassembled, then the :obj:`dict` mapping of data for IPv6
-          reassembly (:term:`ipv6.packet`) will be returned; otherwise, returns :data:`None`.
+          reassembly (:term:`reasm.ipv6.packet`) will be returned; otherwise, returns :data:`None`.
 
     See Also:
-        :class:`pcapkit.foundation.reassembly.ipv6.IPv6Reassembly`
+        :class:`pcapkit.foundation.reassembly.ipv6.IPv6`
 
     """
     ipv6 = getattr(packet, 'ip6', None)  # type: Optional[IP6]
@@ -187,7 +189,7 @@ def ipv6_reassembly(packet: 'Packet', *, count: 'int' = -1) -> 'IP_Packet[IPv6Ad
                 cast('IPv6Address',
                      ipaddress.ip_address(ipv6.dst)),            # destination IP address
                 ipv6.flow,                                       # label
-                RegType_TransType.get(ipv6_frag.nh).name,        # next header field in IPv6 Fragment Header
+                Enum_TransType.get(ipv6_frag.nh).name,        # next header field in IPv6 Fragment Header
             ),
             num=count,                                           # original packet range number
             fo=ipv6_frag.nxt,                                    # fragment offset
@@ -214,10 +216,10 @@ def tcp_reassembly(packet: 'Packet', *, count: 'int' = -1) -> 'TCP_Packet | None
         * If the ``packet`` can be used for TCP reassembly. A packet can be reassembled
           if it contains TCP layer (:class:`dpkt.tcp.TCP`).
         * If the ``packet`` can be reassembled, then the :obj:`dict` mapping of data for TCP
-          reassembly (:term:`tcp.packet`) will be returned; otherwise, returns :data:`None`.
+          reassembly (:term:`reasm.tcp.packet`) will be returned; otherwise, returns :data:`None`.
 
     See Also:
-        :class:`pcapkit.foundation.reassembly.tcp.TCPReassembly`
+        :class:`pcapkit.foundation.reassembly.tcp.TCP`
 
     """
     if hasattr(packet, 'ip'):
@@ -256,7 +258,7 @@ def tcp_reassembly(packet: 'Packet', *, count: 'int' = -1) -> 'TCP_Packet | None
 
 
 def tcp_traceflow(packet: 'Packet', timestamp: 'float', *,
-                  data_link: 'RegType_LinkType', count: 'int' = -1) -> 'TF_Packet | None':
+                  data_link: 'Enum_LinkType', count: 'int' = -1) -> 'TF_TCP_Packet | None':
     """Trace packet flow for TCP.
 
     Args:
@@ -271,10 +273,10 @@ def tcp_traceflow(packet: 'Packet', timestamp: 'float', *,
         * If the ``packet`` can be used for TCP flow tracing. A packet can be reassembled
           if it contains TCP layer (:class:`dpkt.tcp.TCP`).
         * If the ``packet`` can be reassembled, then the :obj:`dict` mapping of data for TCP
-          flow tracing (:term:`trace.packet`) will be returned; otherwise, returns :data:`None`.
+          flow tracing (:term:`trace.tcp.packet`) will be returned; otherwise, returns :data:`None`.
 
     See Also:
-        :class:`pcapkit.foundation.traceflow.TraceFlow`
+        :class:`pcapkit.foundation.traceflow.tcp.TCP`
 
     """
     if hasattr(packet, 'ip'):
@@ -288,7 +290,7 @@ def tcp_traceflow(packet: 'Packet', timestamp: 'float', *,
     if tcp is not None:
         flags = bin(tcp.flags)[2:].zfill(8)
 
-        data = TF_Packet(  # type: ignore[type-var]
+        data = TF_TCP_Packet(  # type: ignore[type-var]
             protocol=data_link,                                         # data link type from global header
             index=count,                                                # frame number
             frame=packet2dict(packet, timestamp, data_link=data_link),  # extracted packet
