@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
-""":mod:`PyShark <pyshark>` Tools
-====================================
+"""PyShark Tools
+===================
 
 :mod:`pcapkit.toolkit.pyshark` contains all you need for
 :mod:`pcapkit` handy usage with `PyShark`_ engine. All
@@ -9,12 +9,18 @@ usable for its caller.
 
 .. _PyShark: https://kiminewt.github.io/pyshark
 
+.. note::
+
+   Due to the lack of functionality of `PyShark`_, some
+   functions of :mod:`pcapkit` may not be available with
+   the `PyShark`_ engine.
+
 """
 import ipaddress
 from typing import TYPE_CHECKING, cast
 
-from pcapkit.const.reg.linktype import LinkType as RegType_LinkType
-from pcapkit.foundation.traceflow import Packet as TF_Packet
+from pcapkit.const.reg.linktype import LinkType as Enum_LinkType
+from pcapkit.foundation.traceflow.data.tcp import Packet as TF_TCP_Packet
 
 if TYPE_CHECKING:
     from typing import Any
@@ -49,7 +55,7 @@ def packet2dict(packet: 'Packet') -> 'dict[str, Any]':
     return dict_
 
 
-def tcp_traceflow(packet: 'Packet') -> 'TF_Packet | None':
+def tcp_traceflow(packet: 'Packet') -> 'TF_TCP_Packet | None':
     """Trace packet flow for TCP.
 
     Args:
@@ -61,10 +67,10 @@ def tcp_traceflow(packet: 'Packet') -> 'TF_Packet | None':
         * If the ``packet`` can be used for TCP flow tracing. A packet can be reassembled
           if it contains TCP layer.
         * If the ``packet`` can be reassembled, then the :obj:`dict` mapping of data for TCP
-          flow tracing (:term:`trace.packet`) will be returned; otherwise, returns :data:`None`.
+          flow tracing (:term:`trace.tcp.packet`) will be returned; otherwise, returns :data:`None`.
 
     See Also:
-        :class:`~pcapkit.foundation.traceflow.TraceFlow`
+        :class:`~pcapkit.foundation.traceflow.tcp.TCP`
 
     """
     if 'IP' in packet:
@@ -77,8 +83,8 @@ def tcp_traceflow(packet: 'Packet') -> 'TF_Packet | None':
     if 'TCP' in packet:
         tcp = cast('Packet', packet.tcp)
 
-        data = TF_Packet(  # type: ignore[type-var]
-            protocol=RegType_LinkType.get(packet.layers[0].layer_name.upper()),  # data link type from global header
+        data = TF_TCP_Packet(  # type: ignore[type-var]
+            protocol=Enum_LinkType.get(packet.layers[0].layer_name.upper()),  # data link type from global header
             index=int(packet.number),                                            # frame number
             frame=packet2dict(packet),                                           # extracted packet
             syn=bool(int(tcp.flags_syn)),                                        # TCP synchronise (SYN) flag
