@@ -1,6 +1,8 @@
 # -*- coding: utf-8 -*-
-""":mod:`Scapy <scapy>` Tools
-================================
+"""Scapy Tools
+=================
+
+.. module:: pcapkit.toolkit.scapy
 
 :mod:`pcapkit.toolkit.scapy` contains all you need for
 :mod:`pcapkit` handy usage with `Scapy`_ engine. All reforming
@@ -18,19 +20,19 @@ import ipaddress
 import time
 from typing import TYPE_CHECKING, cast
 
-from pcapkit.const.reg.linktype import LinkType as RegType_LinkType
-from pcapkit.const.reg.transtype import TransType as RegType_TransType
-from pcapkit.foundation.reassembly.ip import Packet as IP_Packet
-from pcapkit.foundation.reassembly.tcp import Packet as TCP_Packet
-from pcapkit.foundation.traceflow import Packet as TF_Packet
+from pcapkit.const.reg.linktype import LinkType as Enum_LinkType
+from pcapkit.const.reg.transtype import TransType as Enum_TransType
+from pcapkit.foundation.reassembly.data.ip import Packet as IP_Packet
+from pcapkit.foundation.reassembly.data.tcp import Packet as TCP_Packet
+from pcapkit.foundation.traceflow.data.tcp import Packet as TF_TCP_Packet
 from pcapkit.utilities.compat import ModuleNotFoundError  # pylint: disable=redefined-builtin
 from pcapkit.utilities.exceptions import ModuleNotFound, stacklevel
 from pcapkit.utilities.warnings import ScapyWarning, warn
 
 try:
-    import scapy.all as scapy_all
+    import scapy
 except ModuleNotFoundError:
-    scapy_all = None
+    scapy = None
     warn("dependency package 'Scapy' not found",
          ScapyWarning, stacklevel=stacklevel())
 
@@ -39,7 +41,7 @@ if TYPE_CHECKING:
     from typing import Any
 
     from scapy.layers.inet import IP, TCP
-    from scapy.layers.inet6 import IPv6, IPv6ExtHdrFragment
+    from scapy.layers.inet6 import IPv6
     from scapy.packet import Packet
 
 __all__ = [
@@ -61,12 +63,13 @@ def packet2chain(packet: 'Packet') -> 'str':
         ModuleNotFound: If `Scapy`_ is not installed.
 
     """
-    if scapy_all is None:
+    if scapy is None:
         raise ModuleNotFound("No module named 'scapy'", name='scapy')
+    from scapy.packet import NoPayload
 
     chain = [packet.name]
     payload = packet.payload
-    while not isinstance(payload, scapy_all.packet.NoPayload):
+    while not isinstance(payload, NoPayload):
         chain.append(payload.name)
         payload = payload.payload
     return ':'.join(chain)
@@ -85,13 +88,14 @@ def packet2dict(packet: 'Packet') -> 'dict[str, Any]':
         ModuleNotFound: If `Scapy`_ is not installed.
 
     """
-    if scapy_all is None:
+    if scapy is None:
         raise ModuleNotFound("No module named 'scapy'", name='scapy')
+    from scapy.packet import NoPayload
 
     def wrapper(packet: 'Packet') -> 'dict[str, Any]':
         dict_ = packet.fields
         payload = packet.payload
-        if not isinstance(payload, scapy_all.packet.NoPayload):
+        if not isinstance(payload, NoPayload):
             dict_[payload.name] = wrapper(payload)
         return dict_
 
@@ -115,10 +119,10 @@ def ipv4_reassembly(packet: 'Packet', *, count: 'int' = -1) -> 'IP_Packet[IPv4Ad
           if it contains IPv4 layer (:class:`scapy.layers.inet.IP`) and the **DF**
           (:attr:`scapy.layers.inet.IP.flags.DF`) flag is ``False``.
         * If the ``packet`` can be reassembled, then the :obj:`dict` mapping of data for IPv4
-          reassembly (:term:`ipv4.packet`) will be returned; otherwise, returns :data:`None`.
+          reassembly (:term:`reasm.ipv4.packet`) will be returned; otherwise, returns :data:`None`.
 
     See Also:
-        :class:`pcapkit.foundation.reassembly.ipv4.IPv4Reassembly`
+        :class:`pcapkit.foundation.reassembly.ipv4.IPv4`
 
     """
     if 'IP' in packet:
@@ -133,7 +137,7 @@ def ipv4_reassembly(packet: 'Packet', *, count: 'int' = -1) -> 'IP_Packet[IPv4Ad
                 cast('IPv4Address',
                      ipaddress.ip_address(ipv4.dst)),  # destination IP address
                 ipv4.id,                               # identification
-                RegType_TransType.get(ipv4.proto),     # payload protocol type
+                Enum_TransType.get(ipv4.proto),     # payload protocol type
             ),
             num=count,                                 # original packet range number
             fo=ipv4.frag,                              # fragment offset
@@ -161,21 +165,22 @@ def ipv6_reassembly(packet: 'Packet', *, count: 'int' = -1) -> 'IP_Packet[IPv6Ad
           if it contains IPv6 layer (:class:`scapy.layers.inet6.IPv6`) and IPv6 Fragment
           header (:rfc:`2460#section-4.5`, :class:`scapy.layers.inet6.IPv6ExtHdrFragment`).
         * If the ``packet`` can be reassembled, then the :obj:`dict` mapping of data for IPv6
-          reassembly (:term:`ipv6.packet`) will be returned; otherwise, returns :data:`None`.
+          reassembly (:term:`reasm.ipv6.packet`) will be returned; otherwise, returns :data:`None`.
 
     Raises:
         ModuleNotFound: If `Scapy`_ is not installed.
 
     See Also:
-        :class:`pcapkit.foundation.reassembly.ipv6.IPv6Reassembly`
+        :class:`pcapkit.foundation.reassembly.ipv6.IPv6`
 
     """
-    if scapy_all is None:
+    if scapy is None:
         raise ModuleNotFound("No module named 'scapy'", name='scapy')
+    from scapy.layers.inet6 import IPv6ExtHdrFragment
 
     if 'IPv6' in packet:
         ipv6 = cast('IPv6', packet['IPv6'])
-        if scapy_all.IPv6ExtHdrFragment not in ipv6:  # pylint: disable=E1101
+        if IPv6ExtHdrFragment not in ipv6:  # pylint: disable=E1101
             return None                        # dismiss not fragmented packet
         ipv6_frag = cast('IPv6ExtHdrFragment', ipv6['IPv6ExtHdrFragment'])
 
@@ -186,7 +191,7 @@ def ipv6_reassembly(packet: 'Packet', *, count: 'int' = -1) -> 'IP_Packet[IPv6Ad
                 cast('IPv6Address',
                      ipaddress.ip_address(ipv6.dst)),     # destination IP address
                 ipv6.fl,                                  # label
-                RegType_TransType.get(ipv6_frag.nh),      # next header field in IPv6 Fragment Header
+                Enum_TransType.get(ipv6_frag.nh),      # next header field in IPv6 Fragment Header
             ),
             num=count,                                    # original packet range number
             fo=ipv6_frag.offset,                          # fragment offset
@@ -213,10 +218,10 @@ def tcp_reassembly(packet: 'Packet', *, count: 'int' = -1) -> 'TCP_Packet | None
         * If the ``packet`` can be used for TCP reassembly. A packet can be reassembled
           if it contains TCP layer (:class:`scapy.layers.inet.TCP`).
         * If the ``packet`` can be reassembled, then the :obj:`dict` mapping of data for TCP
-          reassembly (:term:`tcp.packet`) will be returned; otherwise, returns :data:`None`.
+          reassembly (:term:`reasm.tcp.packet`) will be returned; otherwise, returns :data:`None`.
 
     See Also:
-        :class:`pcapkit.foundation.reassembly.tcp.TCPReassembly`
+        :class:`pcapkit.foundation.reassembly.tcp.TCP`
 
     """
     if 'IP' in packet:
@@ -253,7 +258,7 @@ def tcp_reassembly(packet: 'Packet', *, count: 'int' = -1) -> 'TCP_Packet | None
     return None
 
 
-def tcp_traceflow(packet: 'Packet', *, count: 'int' = -1) -> 'TF_Packet | None':
+def tcp_traceflow(packet: 'Packet', *, count: 'int' = -1) -> 'TF_TCP_Packet | None':
     """Trace packet flow for TCP.
 
     Args:
@@ -266,18 +271,18 @@ def tcp_traceflow(packet: 'Packet', *, count: 'int' = -1) -> 'TF_Packet | None':
         * If the ``packet`` can be used for TCP flow tracing. A packet can be reassembled
           if it contains TCP layer (:class:`scapy.layers.inet.TCP`).
         * If the ``packet`` can be reassembled, then the :obj:`dict` mapping of data for TCP
-          flow tracing (:term:`trace.packet`) will be returned; otherwise, returns :data:`None`.
+          flow tracing (:term:`trace.tcp.packet`) will be returned; otherwise, returns :data:`None`.
 
     See Also:
-        :class:`pcapkit.foundation.traceflow.TraceFlow`
+        :class:`pcapkit.foundation.traceflow.tcp.TCP`
 
     """
     if 'TCP' in packet:
         ip = cast('IP', packet['IP']) if 'IP' in packet else cast('IPv6', packet['IPv6'])
         tcp = cast('TCP', packet['TCP'])
 
-        data = TF_Packet(  # type: ignore[type-var]
-            protocol=RegType_LinkType.get(packet.name.upper()),  # data link type from global header
+        data = TF_TCP_Packet(  # type: ignore[type-var]
+            protocol=Enum_LinkType.get(packet.name.upper()),  # data link type from global header
             index=count,                                         # frame number
             frame=packet2dict(packet),                           # extracted packet
             syn=bool(tcp.flags.S),                               # TCP synchronise (SYN) flag
