@@ -2,6 +2,8 @@
 """User Defined Exceptions
 =============================
 
+.. module:: pcapkit.utilities.exceptions
+
 :mod:`pcapkit.exceptions` refined built-in exceptions.
 Make it possible to show only user error stack infomation [*]_,
 when exception raised on user's operation.
@@ -19,7 +21,7 @@ import traceback
 from typing import TYPE_CHECKING
 
 from pcapkit.utilities.compat import ModuleNotFoundError  # pylint: disable=redefined-builtin
-from pcapkit.utilities.logging import DEVMODE, logger
+from pcapkit.utilities.logging import DEVMODE, VERBOSE, logger
 
 if TYPE_CHECKING:
     from typing import Any
@@ -33,14 +35,15 @@ __all__ = [
     'DictError', 'ListError', 'TupleError', 'IterableError',        # TypeError
     'IOObjError', 'ProtocolUnbound', 'CallableError',               # TypeError
     'InfoError', 'IPError', 'EnumError', 'ComparisonError',         # TypeError
-    'RegistryError',                                                # TypeError
+    'RegistryError', 'FieldError',                                  # TypeError
     'FormatError', 'UnsupportedCall',                               # AttributeError
     'FileError',                                                    # IOError
     'FileExists',                                                   # FileExistsError
     'FileNotFound',                                                 # FileNotFoundError
     'ProtocolNotFound',                                             # IndexError
     'VersionError', 'IndexNotFound', 'ProtocolError',               # ValueError
-    'EndianError', 'KeyExists',                                     # ValueError
+    'EndianError', 'KeyExists', 'NoDefaultValue',                   # ValueError
+    'FieldValueError', 'SchemaError',                               # ValueError
     'ProtocolNotImplemented', 'VendorNotImplemented',               # NotImplementedError
     'StructError',                                                  # struct.error
     'MissingKeyError', 'FragmentError', 'PacketError',              # KeyError
@@ -97,12 +100,15 @@ class BaseError(Exception):
         # log error
         if not quiet:
             if DEVMODE:
-                logger.error('%s: %s', type(self).__name__, str(self), exc_info=self,
-                             stack_info=True, stacklevel=-stacklevel())
-                # logger.error('%s: %s', type(self).__name__, str(self))
+                logger.error('%s: %s', type(self).__name__, str(self),
+                             exc_info=self if VERBOSE else False,
+                             stack_info=VERBOSE, stacklevel=-stacklevel())
             else:
                 logger.error('%s: %s', type(self).__name__, str(self))
 
+            # logger.error('%s: %s', type(self).__name__, str(self), exc_info=self,
+            #              stack_info=True, stacklevel=-stacklevel())
+
         if not DEVMODE:
             sys.tracebacklimit = 0
         super().__init__(*args, **kwargs)
@@ -193,6 +199,10 @@ class RegistryError(BaseError, TypeError):
     """The argument(s) must be *registry* type."""
 
 
+class FieldError(BaseError, TypeError):
+    """The argument(s) must be *field* type."""
+
+
 ##############################################################################
 # AttributeError session.
 ##############################################################################
@@ -270,6 +280,18 @@ class KeyExists(BaseError, ValueError):
     """Key already exists."""
 
 
+class NoDefaultValue(BaseError, ValueError):
+    """No default value."""
+
+
+class FieldValueError(BaseError, ValueError):
+    """Invalid field value."""
+
+
+class SchemaError(BaseError, ValueError):
+    """Invalid schema."""
+
+
 ##############################################################################
 # NotImplementedError session.
 ##############################################################################
