@@ -2,6 +2,8 @@
 """User Defined Warnings
 ===========================
 
+.. module:: pcapkit.utilities.warnings
+
 :mod:`pcapkit.warnings` refined built-in warnings.
 
 """
@@ -9,7 +11,7 @@ import warnings
 from typing import TYPE_CHECKING
 
 from pcapkit.utilities.exceptions import stacklevel as stacklevel_calculator
-from pcapkit.utilities.logging import DEVMODE, logger
+from pcapkit.utilities.logging import DEVMODE, VERBOSE, logger
 
 if TYPE_CHECKING:
     from typing import Any, Optional, Type, Union
@@ -24,9 +26,12 @@ __all__ = [
     # RuntimeWarning
     'FileWarning', 'LayerWarning', 'ProtocolWarning', 'AttributeWarning',
     'DevModeWarning', 'VendorRequestWarning', 'VendorRuntimeWarning',
+    'UnknownFieldWarning', 'RegistryWarning', 'SchemaWarning', 'InfoWarning',
     # ResourceWarning
     'DPKTWarning', 'ScapyWarning', 'PySharkWarning', 'EmojiWarning',
     'VendorWarning',
+    # DeprecationWarning
+    'DeprecatedFormatWarning',
 ]
 
 
@@ -56,9 +61,13 @@ class BaseWarning(UserWarning):
     def __init__(self, *args: 'Any', **kwargs: 'Any') -> 'None':  # pylint: disable=useless-super-delegation
         # log warning
         if DEVMODE:
-            logger.warning(str(self), exc_info=self)
+            if VERBOSE:
+                logger.warning(str(self), exc_info=self, stack_info=True,
+                            stacklevel=stacklevel_calculator())
+            else:
+                logger.warning(str(self))
         else:
-            logger.warning(str(self))
+            warnings.simplefilter('ignore', type(self))
 
         # warnings.simplefilter('default')
         super().__init__(*args, **kwargs)
@@ -114,6 +123,22 @@ class VendorRuntimeWarning(BaseWarning, RuntimeWarning):
     """Vendor failed during runtime."""
 
 
+class UnknownFieldWarning(BaseWarning, RuntimeWarning):
+    """Unknown field."""
+
+
+class RegistryWarning(BaseWarning, RuntimeWarning):
+    """Registry warning."""
+
+
+class SchemaWarning(BaseWarning, RuntimeWarning):
+    """Schema warning."""
+
+
+class InfoWarning(BaseWarning, RuntimeWarning):
+    """Info class warning."""
+
+
 ##############################################################################
 # ResourceWarning session.
 ##############################################################################
@@ -137,3 +162,12 @@ class EmojiWarning(BaseWarning, ResourceWarning):
 
 class VendorWarning(BaseWarning, ResourceWarning):
     """Warnings on vendor usage."""
+
+
+##############################################################################
+# DeprecationWarning session.
+##############################################################################
+
+
+class DeprecatedFormatWarning(BaseWarning, DeprecationWarning):
+    """Warning on deprecated formats."""
